@@ -32,10 +32,8 @@
 #include "pixmaps/logo.xpm"
 #include "pixmaps/cancel.xpm"
 #include "pixmaps/about_small.xpm"
-#include "pixmaps/panic.xpm"
 
 static GtkWidget *about = NULL;
-static GtkWidget *help = NULL;
 
 static void destroy_about()
 {
@@ -53,7 +51,7 @@ static void version_exit()
 
 static void about_click(GtkWidget *w, gpointer m)
 {
-	open_url_nw(NULL, WEBSITE);
+	open_url(NULL, WEBSITE);
 }
 
 char *name()
@@ -71,6 +69,15 @@ char *version()
 	return VERSION;
 }
 
+gboolean is_applet()
+{
+#ifdef USE_APPLET
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
+
 void show_about(GtkWidget *w, void *null)
 {
 	GtkWidget *vbox;
@@ -84,69 +91,75 @@ void show_about(GtkWidget *w, void *null)
 	GtkWidget *hbox;
 	GtkWidget *button;
 	GtkWidget *text;
+	GtkWidget *sw;
 
 	char abouttitle[45];
 
 	if (!about) {
 
-		about = gtk_window_new(GTK_WINDOW_DIALOG);
-
-		gtk_widget_set_usize(GTK_WIDGET(about), 450, 450);
-
-		g_snprintf(abouttitle, sizeof(abouttitle), _("About GAIM v%s"), VERSION);
+		GAIM_DIALOG(about);
+		gtk_window_set_default_size(GTK_WINDOW(about), 450, -1);
+		g_snprintf(abouttitle, sizeof(abouttitle), _("About Gaim v%s"), VERSION);
 		gtk_window_set_title(GTK_WINDOW(about), abouttitle);
 		gtk_window_set_wmclass(GTK_WINDOW(about), "about", "Gaim");
 		gtk_window_set_policy(GTK_WINDOW(about), FALSE, TRUE, TRUE);
-
 		gtk_widget_realize(about);
 		aol_icon(about->window);
 
 		vbox = gtk_vbox_new(FALSE, 5);
 		gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 		gtk_container_add(GTK_CONTAINER(about), vbox);
+		gtk_widget_show(vbox);
 
-		frame = gtk_frame_new("Gaim " VERSION);
+		frame = gtk_frame_new("Gaim v" VERSION);
 		gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
+		gtk_widget_show(frame);
 
 		fbox = gtk_vbox_new(FALSE, 5);
 		gtk_container_set_border_width(GTK_CONTAINER(fbox), 5);
 		gtk_container_add(GTK_CONTAINER(frame), fbox);
+		gtk_widget_show(fbox);
 
 		/* Left side, TOP */
 		style = gtk_widget_get_style(about);
 		pm = gdk_pixmap_create_from_xpm_d(about->window, &bm,
-						  &style->bg[GTK_STATE_NORMAL], (gchar **)aol_logo);
+						  &style->bg[GTK_STATE_NORMAL], (gchar **)gaim_logo_xpm);
 		pixmap = gtk_pixmap_new(pm, bm);
-
 		gdk_pixmap_unref(pm);
 		gdk_bitmap_unref(bm);
-
 		gtk_box_pack_start(GTK_BOX(fbox), pixmap, FALSE, FALSE, 0);
+		gtk_widget_show(pixmap);
 
 		label =
 		    gtk_label_new(
-				  _("GAIM is a client that supports AOL's Instant Messenger protocol. It is written\n"
+				  _("Gaim is a client that supports AOL's Instant Messenger protocol. It is written\n"
 				   "using Gtk+ and is licensed under the GPL.\n\n" "URL: " WEBSITE "\n\n"
 				   "IRC: #gaim on irc.openprojects.net"));
 
 		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-
 		gtk_box_pack_start(GTK_BOX(fbox), label, TRUE, TRUE, 0);
+		gtk_widget_show(label);
 
 		text = gtk_text_new(NULL, NULL);
 
-		gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL,
-				_("Rob Flynn (maintainer)               rob@marko.net\nEric Warmenhoven (lead coder)  warmenhoven@yahoo.com\n\nBenjamin Miller\nDecklin Foster\nJim Duchek\nMark Spencer (original author)   markster@marko.net"),
-				198);
+		gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL,
+                   _("Active Developers\n====================\nRob Flynn (maintainer) [ rob@marko.net ]\nSean Egan (coder)      [ bj91704@binghamton.edu ]\n\nCrazy Patch Writers\n===================\nBenjamin Miller\nDecklin Foster\nNathan Walp\nMark Doliner\n\nRetired Developers\n===================\nJim Duchek\nEric Warmenhoven                   [ warmenhoven@yahoo.com ]\nMark Spencer (original author)   [ markster@marko.net ]"), 394);
 
+		sw = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+
+		gtk_container_add(GTK_CONTAINER(sw), text);
+		gtk_widget_set_usize(GTK_WIDGET(sw), -1, 150);
+		gtk_widget_show(sw);
+		
+		gtk_box_pack_start(GTK_BOX(fbox), sw, TRUE, TRUE, 0);
 		gtk_widget_show(text);
 
-		gtk_box_pack_start(GTK_BOX(fbox), text, TRUE, TRUE, 0);
-
 		/* Close Button */
 
 		hbox = gtk_hbox_new(FALSE, 5);
 		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+		gtk_widget_show(hbox);
 
 		button = picture_button(about, _("Close"), cancel_xpm);
 		gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
@@ -171,79 +184,10 @@ void show_about(GtkWidget *w, void *null)
 		button = picture_button(about, _("Web Site"), about_small_xpm);
 		gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(about_click), NULL);
-
-		if (misc_options & OPT_MISC_COOL_LOOK)
-			gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	}
 
 	/* Let's give'em something to talk about -- woah woah woah */
-	gtk_widget_show_all(about);
+	gtk_widget_show(about);
 
-}
-
-static void destroy_help()
-{
-	if (help)
-		gtk_widget_destroy(help);
-	help = NULL;
-}
-
-void gaim_help(GtkWidget *w, void *x)
-{
-	GtkWidget *vbox;
-	GdkPixmap *pm;
-	GdkBitmap *bm;
-	GtkWidget *pix;
-	GtkWidget *label;
-	GtkWidget *hbox;
-	GtkWidget *close;
-
-	if (!help) {
-
-		help = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_title(GTK_WINDOW(help), "Gaim Help");
-		gtk_window_set_wmclass(GTK_WINDOW(help), "help", "Gaim");
-		gtk_window_set_policy(GTK_WINDOW(help), FALSE, TRUE, TRUE);
-		gtk_widget_realize(help);
-		aol_icon(help->window);
-		gtk_signal_connect(GTK_OBJECT(help), "destroy",
-				   GTK_SIGNAL_FUNC(destroy_help), GTK_OBJECT(help));
-		gtk_widget_set_usize(help, 315, -1);
-
-		vbox = gtk_vbox_new(FALSE, 10);
-		gtk_container_add(GTK_CONTAINER(help), vbox);
-
-		hbox = gtk_hbox_new(FALSE, 10);
-		gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
-
-		close = picture_button(help, _("Close"), cancel_xpm);
-		gtk_box_pack_end(GTK_BOX(hbox), close, FALSE, FALSE, 5);
-		gtk_signal_connect(GTK_OBJECT(close), "clicked", GTK_SIGNAL_FUNC(destroy_help), NULL);
-
-		pm = gdk_pixmap_create_from_xpm_d(help->window, &bm, NULL, panic_xpm);
-		pix = gtk_pixmap_new(pm, bm);
-		gdk_pixmap_unref(pm);
-		gdk_bitmap_unref(bm);
-		gtk_box_pack_start(GTK_BOX(vbox), pix, FALSE, FALSE, 5);
-
-		label = gtk_label_new(_("Relax, help is just around the corner. "
-					"The first thing you'll need to do is get an AIM "
-					"account; you can get one from http://aim.aol.com/. "
-					"Just click the button that says \"New Users\" and "
-					"you can create an account that way. Once you have "
-					"your account, enter the username and password into "
-					"the login window that comes up when you start Gaim, "
-					"and click the Signon button. Once you're online, "
-					"you can talk to one of the Gaim developers for more "
-					"assistance; their contact information is in the "
-					"AUTHORS file in the Gaim source, or at "
-					WEBSITE "contact.shtml. If you can't get online and "
-					"still need more assistance, feel free to email us "
-					"at gaim@marko.net. Thanks for using Gaim!"));
-		gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-		gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-		gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 5);
-	}
-
-	gtk_widget_show_all(help);
+	gdk_window_raise(about->window);
 }
