@@ -33,6 +33,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <math.h>
 #include <time.h>
 #include <unistd.h>
@@ -56,9 +57,7 @@
 #include "pixmaps/plugins_small.xpm"
 #endif
 #include "pixmaps/prefs_small.xpm"
-#ifdef NO_MULTI
 #include "pixmaps/search_small.xpm"
-#endif
 #ifdef USE_APPLET
 #include "pixmaps/close_small.xpm"
 #else
@@ -74,6 +73,10 @@
 #include "pixmaps/gnome_add.xpm"
 #include "pixmaps/gnome_remove.xpm"
 #include "pixmaps/group.xpm"
+#include "pixmaps/logout_menu.xpm"
+
+#include "pixmaps/arrow_down.xpm"
+#include "pixmaps/arrow_right.xpm"
 
 static GtkTooltips *tips;
 static GtkAccelGroup *accel;
@@ -91,8 +94,8 @@ GtkWidget *buddies;
 void BuddyTickerLogonTimeout(gpointer data);
 void BuddyTickerLogoutTimeout(gpointer data);
 
-/* Predefine some functions */
-static void new_bp_callback(GtkWidget *w, char *name);
+typedef struct _GtkTreePixmaps GtkTreePixmaps;
+
 
 struct buddy_show {
 	GtkWidget *item;
@@ -117,6 +120,8 @@ struct group_show {
 };
 static GSList *shows = NULL;
 
+/* Predefine some functions */
+static void new_bp_callback(GtkWidget *w, struct buddy *bs);
 static struct group_show *find_group_show(char *group);
 static struct buddy_show *find_buddy_show(struct group_show *gs, char *name);
 static int group_number(char *group);
@@ -126,6 +131,7 @@ static struct buddy_show *new_buddy_show(struct group_show *gs, struct buddy *bu
 static void remove_buddy_show(struct group_show *gs, struct buddy_show *bs);
 static struct group_show *find_gs_by_bs(struct buddy_show *b);
 static void update_num_group(struct group_show *gs);
+static void update_idle_time(struct buddy_show *bs);
 
 void handle_group_rename(struct group *g, char *prevname)
 {
@@ -164,9 +170,9 @@ void handle_group_rename(struct group *g, char *prevname)
 					g_free(bs);
 				}
 				if ((bs = find_buddy_show(new_gs, b->name)) == NULL) {
-					if (*g->gc->prpl->list_icon) {
+					if (g->gc->prpl->list_icon) {
 						bs = new_buddy_show(new_gs, b,
-								    (*g->gc->prpl->list_icon)(b->uc));
+								    g->gc->prpl->list_icon(b->uc));
 					} else {
 						bs = new_buddy_show(new_gs, b, (char **)no_icon_xpm);
 					}
@@ -217,9 +223,9 @@ void handle_group_rename(struct group *g, char *prevname)
 						g_free(bs->name);
 						g_free(bs);
 					}
-					if (*g->gc->prpl->list_icon) {
+					if (g->gc->prpl->list_icon) {
 						bs = new_buddy_show(new_gs, b,
-								    (*g->gc->prpl->list_icon)(b->uc));
+								    g->gc->prpl->list_icon(b->uc));
 					} else {
 						bs = new_buddy_show(new_gs, b, (char **)no_icon_xpm);
 					}
@@ -240,12 +246,19 @@ void handle_group_rename(struct group *g, char *prevname)
 
 void handle_buddy_rename(struct buddy *b, char *prevname)
 {
+	struct conversation *cnv;
 	struct buddy_show *bs;
 	struct group_show *gs;
 	struct group *g;
 	GtkCTreeNode *c;
 	char buf[256];
 
+	if (!strcmp(b->show, prevname))
+		g_snprintf(b->show, sizeof(b->show), "%s", b->name);
+
+	/* well you shouldn't be calling this if nothing changed. duh. */
+	do_export(b->gc);
+
 	c = gtk_ctree_find_by_row_data(GTK_CTREE(edittree), NULL, b);
 	if (strcmp(b->show, b->name))
 		g_snprintf(buf, sizeof(buf), "%s (%s)", b->name, b->show);
@@ -253,6 +266,9 @@ void handle_buddy_rename(struct buddy *b, char *prevname)
 		g_snprintf(buf, sizeof(buf), "%s", b->name);
 	gtk_ctree_node_set_text(GTK_CTREE(edittree), c, 0, buf);
 
+	if ((cnv = find_conversation(b->name)) != NULL)
+		set_convo_title(cnv);
+
 	gs = find_group_show(prevname);
 
 	g = find_group_by_buddy(b->gc, b->name);
@@ -282,8 +298,10 @@ void handle_buddy_rename(struct buddy *b, char *prevname)
 			g_free(bs->name);
 			g_free(bs);
 		}
+		update_num_group(gs);
 	} else {
 		gtk_label_set_text(GTK_LABEL(bs->label), b->show);
+		update_idle_time(bs);
 	}
 }
 
@@ -453,7 +471,7 @@ static void update_num_group(struct group_show *gs)
 	gtk_label_set_text(GTK_LABEL(gs->label), buf);
 }
 
-void update_num_groups()
+void update_num_groups(void)
 {
 	GSList *s = shows;
 	struct group_show *g;
@@ -486,6 +504,21 @@ void update_button_pix()
 	}
 }
 
+void set_blist_tab()
+{
+	GtkWidget *blist_notebook;
+	if (!buddypane)
+		return;
+	
+	blist_notebook = buddypane->parent;  /* The "Online" Page */
+
+	debug_printf("blist_options = %d\n", blist_options);
+	if((blist_options & OPT_BLIST_BOTTOM_TAB))
+		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(blist_notebook), GTK_POS_BOTTOM);
+	else
+		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(blist_notebook), GTK_POS_TOP);
+
+}
 
 
 #ifdef USE_APPLET
@@ -498,15 +531,17 @@ gint applet_destroy_buddy(GtkWidget *widget, GdkEvent *event, gpointer *data)
 
 #endif
 
-void handle_click_group(GtkWidget *widget, GdkEventButton *event, struct group *g)
+static int handle_click_group(GtkWidget *widget, GdkEventButton *event, struct group *g)
 {
 	if (event->type == GDK_2BUTTON_PRESS) {
 		if (GTK_TREE_ITEM(widget)->expanded)
 			gtk_tree_item_collapse(GTK_TREE_ITEM(widget));
 		else
 			gtk_tree_item_expand(GTK_TREE_ITEM(widget));
-	} else if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
 void pressed_im_bud(GtkWidget *widget, struct buddy *b)
@@ -583,10 +618,10 @@ static void menu_click(GtkObject *obj, char *who)
 	g_list_free(first);
 }
 
-void handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct buddy_show *b)
+static int handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct buddy_show *b)
 {
 	if (!b->connlist)
-		return;
+		return FALSE;
 	if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
 		struct conversation *c;
 
@@ -623,7 +658,8 @@ void handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct buddy_s
 
 		button = gtk_menu_item_new_with_label(_("Add Buddy Pounce"));
 		gtk_signal_connect(GTK_OBJECT(button), "activate",
-				   GTK_SIGNAL_FUNC(new_bp_callback), b->name);
+				   GTK_SIGNAL_FUNC(new_bp_callback),
+				   cn ? find_buddy(cn->data, b->name) : NULL);
 		gtk_menu_append(GTK_MENU(menu), button);
 		gtk_widget_show(button);
 
@@ -637,7 +673,7 @@ void handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct buddy_s
 			while (cn) {
 				g = (struct gaim_connection *)cn->data;
 				if (g->prpl->buddy_menu) {
-					GList *mo = (*g->prpl->buddy_menu)(g, b->name);
+					GList *mo = g->prpl->buddy_menu(g, b->name);
 
 					menuitem = gtk_menu_item_new_with_label(g->username);
 					gtk_menu_append(GTK_MENU(menu), menuitem);
@@ -666,7 +702,7 @@ void handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct buddy_s
 		} else {
 			g = (struct gaim_connection *)cn->data;
 			if (g->prpl->buddy_menu) {
-				GList *mo = (*g->prpl->buddy_menu)(g, b->name);
+				GList *mo = g->prpl->buddy_menu(g, b->name);
 
 				while (mo) {
 					struct proto_buddy_menu *pbm = mo->data;
@@ -698,11 +734,21 @@ void handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct buddy_s
 			 show_ee_dialog(2);
 		else if (!g_strcasecmp("ewarmenhoven", normalize (b->name)))
 			 show_ee_dialog(3);
+		else if (!g_strcasecmp("markster97", normalize (b->name)))
+			 show_ee_dialog(4);
+		else if (!g_strcasecmp("seanegn", normalize (b->name)))
+			show_ee_dialog(5);
+		else if (!g_strcasecmp("chipx86", normalize (b->name)))
+			show_ee_dialog(6);
+		else if (!g_strcasecmp("kingant", normalize (b->name)))
+			show_ee_dialog(7);
 
 	} else {
 
 		/* Anything for other buttons? :) */
 	}
+
+	return FALSE;
 }
 
 static void un_alias(GtkWidget *a, struct buddy *b)
@@ -713,6 +759,7 @@ static void un_alias(GtkWidget *a, struct buddy *b)
 	GtkCTreeNode *node = gtk_ctree_find_by_row_data(GTK_CTREE(edittree), NULL, b);
 	g_snprintf(b->show, sizeof(b->show), "%s", b->name);
 	gtk_ctree_node_set_text(GTK_CTREE(edittree), node, 0, b->name);
+	serv_alias_buddy(b);
 	if (gs)
 		bs = find_buddy_show(gs, b->name);
 	if (bs)
@@ -729,10 +776,10 @@ static gboolean click_edit_tree(GtkWidget *widget, GdkEventButton *event, gpoint
 	GtkWidget *button;
 
 	if (event->button != 3 || event->type != GDK_BUTTON_PRESS)
-		return TRUE;
+		return FALSE;
 
 	if (!gtk_clist_get_selection_info(GTK_CLIST(edittree), event->x, event->y, &row, &column))
-		return TRUE;
+		return FALSE;
 
 	node = gtk_ctree_node_nth(GTK_CTREE(edittree), row);
 	type = gtk_ctree_node_get_row_data(GTK_CTREE(edittree), node);
@@ -747,6 +794,8 @@ static gboolean click_edit_tree(GtkWidget *widget, GdkEventButton *event, gpoint
 		gtk_widget_show(button);
 
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
+
+		return TRUE;
 	} else if (*type == EDIT_BUDDY) {
 		struct buddy *b = (struct buddy *)type;
 		menu = gtk_menu_new();
@@ -777,7 +826,7 @@ static gboolean click_edit_tree(GtkWidget *widget, GdkEventButton *event, gpoint
 
 		button = gtk_menu_item_new_with_label(_("Add Buddy Pounce"));
 		gtk_signal_connect(GTK_OBJECT(button), "activate",
-				   GTK_SIGNAL_FUNC(new_bp_callback), b->name);
+				   GTK_SIGNAL_FUNC(new_bp_callback), b);
 		gtk_menu_append(GTK_MENU(menu), button);
 		gtk_widget_show(button);
 
@@ -788,12 +837,28 @@ static gboolean click_edit_tree(GtkWidget *widget, GdkEventButton *event, gpoint
 		gtk_widget_show(button);
 
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
+
+		return TRUE;
 	}
 
-	return TRUE;
+	return FALSE;
 }
 
 
+/*
+ * Find and remove CTree node associated with buddylist entry
+ */
+static void ui_remove_buddy_node(struct group *rem_g, struct buddy *rem_b)
+{
+	GtkCTreeNode *gnode = NULL, *bnode;
+
+	if((gnode = gtk_ctree_find_by_row_data(GTK_CTREE(edittree), NULL, rem_g)) != NULL &&
+		(bnode = gtk_ctree_find_by_row_data(GTK_CTREE(edittree), gnode, rem_b)) != NULL)
+	{
+		gtk_ctree_remove_node(GTK_CTREE(edittree), bnode);
+	}
+}
+
 void ui_remove_buddy(struct gaim_connection *gc, struct group *rem_g, struct buddy *rem_b)
 {
 	struct conversation *c;
@@ -821,19 +886,22 @@ void ui_remove_buddy(struct gaim_connection *gc, struct group *rem_g, struct bud
 						gtk_tree_remove_item(GTK_TREE(buddies), gs->item);
 						g_free(gs->name);
 						g_free(gs);
-					} else
-						update_num_group(gs);
-				} else
-					update_num_group(gs);
-			} else
-				update_num_group(gs);
-		} else
+						gs = NULL;
+					}
+				}
+			}
+		}
+		if (gs)
 			update_num_group(gs);
 	}
 
 	c = find_conversation(rem_b->name);
 	if (c)
 		update_buttons_by_protocol(c);
+
+	/* Remove CTree node for buddy */
+	ui_remove_buddy_node(rem_g, rem_b);
+
 }
 
 void ui_remove_group(struct gaim_connection *gc, struct group *rem_g)
@@ -944,16 +1012,18 @@ void redo_buddy_list()
 					if (!bs) {
 						if (gc->prpl->list_icon)
 							bs = new_buddy_show(gs, b,
-									    (*gc->prpl->list_icon)(b->
+									    gc->prpl->list_icon(b->
 												   uc));
 						else
 							bs = new_buddy_show(gs, b, (char **)no_icon_xpm);
 					}
 					bs->connlist = g_slist_append(bs->connlist, gc);
+					update_num_group(gs);
 				}
 			}
 		}
 	}
+	update_idle_times();
 }
 
 static void edit_tree_move(GtkCTree *ctree, GtkCTreeNode *child, GtkCTreeNode *parent,
@@ -985,6 +1055,7 @@ static void edit_tree_move(GtkCTree *ctree, GtkCTreeNode *child, GtkCTreeNode *p
 				connections = g_slist_prepend(connections, gc);
 		} else
 			connections = g_slist_append(connections, gc);
+		redo_convo_menus(); /* this is evil */
 	} else if (*ctype == EDIT_BUDDY) {
 		/* we moved a buddy. hopefully we just changed groups or positions or something.
 		 * if we changed connections, we copy the buddy to the new connection. if the new
@@ -992,6 +1063,7 @@ static void edit_tree_move(GtkCTree *ctree, GtkCTreeNode *child, GtkCTreeNode *p
 		 * we change the group that the buddy is in */
 		struct group *old_g, *new_g = (struct group *)ptype;
 		struct buddy *s = NULL, *buddy = (struct buddy *)ctype;
+		gboolean add = FALSE;
 		int pos;
 
 		if (buddy->gc != new_g->gc) {
@@ -1008,7 +1080,7 @@ static void edit_tree_move(GtkCTree *ctree, GtkCTreeNode *child, GtkCTreeNode *p
 				og->members = g_slist_remove(og->members, a);
 			} else {
 				/* we don't have this buddy yet; let's add him */
-				serv_add_buddy(new_g->gc, buddy->name);
+				add = TRUE;
 			}
 		}
 
@@ -1028,6 +1100,16 @@ static void edit_tree_move(GtkCTree *ctree, GtkCTreeNode *child, GtkCTreeNode *p
 		} else
 			new_g->members = g_slist_append(new_g->members, buddy);
 
+		/*
+		 * we do the add after it's added locally so that prpls can find it if necessary
+		 * JFIXME: Er, shouldn't the buddy be removed from the old server, as well?
+		 */
+		if (add) {
+			serv_add_buddy(new_g->gc, buddy->name);
+		} else {
+			serv_move_buddy(buddy, old_g, new_g);
+		}
+
 		do_export(buddy->gc);
 		if (buddy->gc != new_g->gc) {
 			do_export(new_g->gc);
@@ -1098,7 +1180,37 @@ static void edit_tree_move(GtkCTree *ctree, GtkCTreeNode *child, GtkCTreeNode *p
 	update_num_groups();
 }
 
+void
+create_prpl_icon(GtkWidget *widget, struct gaim_connection *gc,
+				 GdkPixmap **pixmap, GdkBitmap **mask)
+{
+    /* This whole thing is a hack--but it looks nice.
+     * Probably should have a prpl->icon(struct gaim_connection *) to
+     * do this. */
+	GtkStyle *style;
+	char **xpm = NULL; 
 
+	if (widget == NULL || gc == NULL || pixmap == NULL || mask == NULL)
+		return;
+	
+	style = gtk_widget_get_style( widget );
+	
+	if (gc->prpl->list_icon) {
+		if (gc->prpl->protocol ==  PROTO_OSCAR) { 
+			if (isdigit(*gc->username)) {
+				xpm = gc->prpl->list_icon(0);
+			} else {
+				xpm = gc->prpl->list_icon(0x10);
+			}
+		} else { 
+			xpm = gc->prpl->list_icon (0);
+		}
+	}
+	if (xpm == NULL)
+		xpm = (char **)no_icon_xpm;
+	
+	*pixmap = gdk_pixmap_create_from_xpm_d(widget->window, mask, &style->bg[GTK_STATE_NORMAL], xpm);
+}
 
 void build_edit_tree()
 {
@@ -1122,10 +1234,18 @@ void build_edit_tree()
 		z = (struct gaim_connection *)con->data;
 
 		if (g_slist_length(connections) > 1) {
+			GdkPixmap *pixmap;
+			GdkBitmap *mask;
+
 			text[0] = z->username;
 
+			create_prpl_icon(blist, z, &pixmap, &mask);
+
 			c = gtk_ctree_insert_node(GTK_CTREE(edittree), NULL,
-						  NULL, text, 5, NULL, NULL, NULL, NULL, 0, 1);
+						  NULL, text, 3, pixmap, mask, pixmap, mask, 0, 1);
+
+			gdk_pixmap_unref (pixmap);
+			gdk_bitmap_unref (mask);
 
 			gtk_ctree_node_set_row_data(GTK_CTREE(edittree), c, z);
 		} else
@@ -1134,6 +1254,7 @@ void build_edit_tree()
 		grp = z->groups;
 
 		while (grp) {
+			
 			g = (struct group *)grp->data;
 
 			text[0] = g->name;
@@ -1238,9 +1359,8 @@ static void do_del_buddy(GtkWidget *w, GtkCTree *ctree)
 			b = (struct buddy *)type;
 			g = find_group_by_buddy(b->gc, b->name);
 			gct = b->gc;
-			serv_remove_buddy(b->gc, b->name);
+			serv_remove_buddy(b->gc, b->name, g->name);
 			remove_buddy(b->gc, g, b);
-			gtk_ctree_remove_node(GTK_CTREE(edittree), node);
 			do_export(gct);
 		} else if (*type == EDIT_GROUP) {
 			struct gaim_connection *gc = ((struct group *)type)->gc;
@@ -1269,6 +1389,7 @@ void do_quit()
 	/* first we tell those who have requested it we're quitting */
 	plugin_event(event_quit, 0, 0, 0, 0);
 
+	signoff_all();
 #ifdef GAIM_PLUGINS
 	/* then we remove everyone in a mass suicide */
 	remove_all_plugins();
@@ -1307,7 +1428,7 @@ void add_buddy_callback(GtkWidget *widget, void *dummy)
 			gc = (struct gaim_connection *)type;
 		}
 	}
-	show_add_buddy(gc, NULL, grp);
+	show_add_buddy(gc, NULL, grp, NULL);
 
 }
 
@@ -1354,6 +1475,8 @@ static void im_callback(GtkWidget *widget, GtkTree *tree)
 	} else {
 		gdk_window_raise(c->window->window);
 	}
+
+	set_convo_gc(c, b->connlist->data);
 }
 
 
@@ -1449,15 +1572,32 @@ void do_pounce(struct gaim_connection *gc, char *name, int when)
 
 				set_convo_gc(c, u->gc);
 			}
+			if (b->options & OPT_POUNCE_NOTIFY) {
+				char tmp[1024];
+
+				/* I know the line below is really ugly. I only did it this way
+				 * because I thought it'd be funny :-) */
+
+				g_snprintf(tmp, sizeof(tmp), "%s has %s", name, 
+					   (b->options & OPT_POUNCE_TYPING) ? "started typing to you" :
+					   (b->options & OPT_POUNCE_SIGNON) ? "signed on" : 
+					   (b->options & OPT_POUNCE_UNIDLE) ? "returned from being idle" : 
+					   "returned from being away");
+				
+				do_error_dialog(tmp, _("Buddy Pounce"));
+			}
 			if (b->options & OPT_POUNCE_SEND_IM) {
-				c = find_conversation(name);
-				if (c == NULL)
-					c = new_conversation(name);
+				if (strlen(b->message) > 0) {
+					c = find_conversation(name);
 
-				set_convo_gc(c, u->gc);
+					if (c == NULL)
+						c = new_conversation(name);
 
-				write_to_conv(c, b->message, WFLAG_SEND, NULL, time((time_t) NULL));
-				serv_send_im(u->gc, name, b->message, 0);
+					set_convo_gc(c, u->gc);
+
+					write_to_conv(c, b->message, WFLAG_SEND, NULL, time(NULL), -1);
+					serv_send_im(u->gc, name, b->message, -1, 0);
+				}
 			}
 			if (b->options & OPT_POUNCE_COMMAND) {
 				int pid = fork();
@@ -1470,15 +1610,13 @@ void do_pounce(struct gaim_connection *gc, char *name, int when)
 					args[3] = NULL;
 					execvp(args[0], args);
 					_exit(0);
-				} else if (pid > 0) {
-					gtk_timeout_add(100, (GtkFunction)clean_pid, NULL);
 				}
 			}
 			if (b->options & OPT_POUNCE_SOUND) {
 				if (strlen(b->sound))
 					play_file(b->sound);
 				else
-					play_sound(POUNCE_DEFAULT);
+					play_sound(SND_POUNCE_DEFAULT);
 			}
 
 			if (!(b->options & OPT_POUNCE_SAVE))
@@ -1489,11 +1627,20 @@ void do_pounce(struct gaim_connection *gc, char *name, int when)
 	g_free(who);
 }
 
-static void new_bp_callback(GtkWidget *w, char *name)
+static void new_bp_callback(GtkWidget *w, struct buddy *b)
 {
-	show_new_bp(name);
+	if (b)
+		show_new_bp(b->name, b->gc, b->idle, b->uc & UC_UNAVAILABLE, NULL);
+	else
+		show_new_bp(NULL, NULL, 0, 0, NULL);
 }
 
+static void edit_bp_callback(GtkWidget *w, struct buddy_pounce *b)
+{
+  show_new_bp(NULL, NULL, 0, 0, b);
+}
+
+static GtkTooltips *bp_tooltip = NULL;
 void do_bp_menu()
 {
 	GtkWidget *menuitem, *mess, *messmenu;
@@ -1504,6 +1651,10 @@ void do_bp_menu()
 	struct buddy_pounce *b;
 	GList *bp = buddy_pounces;
 
+	/* Tooltip for editing bp's */
+	if(!bp_tooltip)
+		bp_tooltip = gtk_tooltips_new();
+
 	l = gtk_container_children(GTK_CONTAINER(bpmenu));
 
 	while (l) {
@@ -1559,8 +1710,9 @@ void do_bp_menu()
 
 		mess = gtk_menu_item_new_with_label(b->message);
 		gtk_menu_append(GTK_MENU(messmenu), mess);
+		gtk_tooltips_set_tip(bp_tooltip, GTK_WIDGET(mess), _("[Click to edit]"), NULL);
 		gtk_widget_show(mess);
-
+		gtk_signal_connect(GTK_OBJECT(mess), "activate", GTK_SIGNAL_FUNC(edit_bp_callback), b);
 		bp = bp->next;
 
 	}
@@ -1666,26 +1818,70 @@ static int buddy_number(char *group, char *buddy)
 	return -1;
 }
 
+
+
 static struct group_show *new_group_show(char *group)
 {
 	struct group_show *g = g_new0(struct group_show, 1);
 	int pos = group_number(group);
+	GdkPixmap *pm;
+	GdkBitmap *bm;
+	GtkStyle *style;
+	GtkStyle *style2;
 
 	g->name = g_strdup(group);
 
 	g->item = gtk_tree_item_new();
-	gtk_tree_insert(GTK_TREE(buddies), g->item, pos);
+
 	gtk_signal_connect(GTK_OBJECT(g->item), "button_press_event",
 			   GTK_SIGNAL_FUNC(handle_click_group), g);
+
+	gtk_tree_insert(GTK_TREE(buddies), g->item, pos);
+
 	gtk_widget_show(g->item);
 
 	g->label = gtk_label_new(group);
 	gtk_misc_set_alignment(GTK_MISC(g->label), 0.0, 0.5);
-	gtk_container_add(GTK_CONTAINER(g->item), g->label);
 	gtk_widget_show(g->label);
 
+	gtk_container_add(GTK_CONTAINER(g->item), g->label);
+
 	shows = g_slist_insert(shows, g, pos);
-	update_num_groups(g);
+
+	/* Rob does drugs - this is still evil, damn you becausse I SAID SO! */
+
+	pm = gdk_pixmap_create_from_xpm_d(g->item->window, 
+		&bm, NULL, arrow_down_xpm);
+
+	gtk_pixmap_set(GTK_PIXMAP(GTK_TREE_ITEM(g->item)->minus_pix_widget), 
+		pm, bm);
+
+	gdk_pixmap_unref(pm);
+	gdk_bitmap_unref(bm);
+
+	pm = gdk_pixmap_create_from_xpm_d(buddies->window, 
+		&bm, NULL, arrow_right_xpm);
+
+	gtk_pixmap_set(GTK_PIXMAP(GTK_TREE_ITEM(g->item)->plus_pix_widget), 
+		pm, bm);
+
+	gdk_pixmap_unref(pm);
+	gdk_bitmap_unref(bm);
+
+//	style = gtk_widget_get_style(GTK_TREE_ITEM(g->item)->pixmaps_box);
+	style2 = gtk_style_copy(gtk_widget_get_style(g->item));
+	style = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(g->label)));
+
+	style->bg[0] = style2->base[0];
+	gtk_widget_set_style(GTK_TREE_ITEM(g->item)->pixmaps_box, style);
+
+	gtk_style_unref(style);
+	gtk_style_unref(style2);
+
+	/* bad drugs */
+
+	update_num_group(g);
+	
 	return g;
 }
 
@@ -1778,6 +1974,26 @@ static struct group_show *find_gs_by_bs(struct buddy_show *b)
 	return g;
 }
 
+void hide_buddy_list() {
+#ifdef USE_APPLET
+	applet_destroy_buddy(NULL, NULL, NULL);
+#else
+	XIconifyWindow(GDK_DISPLAY(),
+			GDK_WINDOW_XWINDOW(blist->window),
+			((_XPrivDisplay)GDK_DISPLAY())->default_screen);
+#endif
+}
+
+void unhide_buddy_list() {
+#ifdef USE_APPLET
+	if (!applet_buddy_show) {
+		applet_buddy_show = TRUE;
+		createOnlinePopup();
+	}
+#endif /* USE_APPLET */
+	gdk_window_show(blist->window);
+}
+
 static gint log_timeout(struct buddy_show *b)
 {
 	/* this part is really just a bad hack because of a bug I can't find */
@@ -1845,12 +2061,12 @@ static gint log_timeout(struct buddy_show *b)
 	return 0;
 }
 
-static char *caps_string(gushort caps)
+static char *caps_string(guint caps)
 {
 	static char buf[256], *tmp;
 	int count = 0, i = 0;
-	gushort bit = 1;
-	while (bit <= 0x80) {
+	guint bit = 1;
+	while (bit <= 0x10000) {
 		if (bit & caps) {
 			switch (bit) {
 			case 0x1:
@@ -1872,11 +2088,36 @@ static char *caps_string(gushort caps)
 				tmp = _("Send File");
 				break;
 			case 0x40:
+			case 0x200:
 				tmp = _("Games");
 				break;
 			case 0x80:
 				tmp = _("Stocks");
 				break;
+			case 0x100:
+				tmp = _("Send Buddy List");
+				break;
+			case 0x400:
+				tmp = _("EveryBuddy Bug");
+				break;
+			case 0x800:
+				tmp = _("AP User");
+				break;
+			case 0x1000:
+				tmp = _("ICQ RTF");
+				break;
+			case 0x2000:
+				tmp = _("Nihilist");
+				break;
+			case 0x4000:
+				tmp = _("ICQ Server Relay");
+				break;
+			case 0x8000:
+				tmp = _("ICQ Unknown");
+				break;
+			case 0x10000:
+				tmp = _("Trillian Encryption");
+				break;
 			default:
 				tmp = NULL;
 				break;
@@ -1900,12 +2141,15 @@ static void update_idle_time(struct buddy_show *bs)
 	time_t t;
 	int ihrs, imin;
 	struct buddy *b;
+	GtkStyle *style;
 
 	char infotip[2048];
 	char warn[256];
 	char caps[256];
 	char *sotime = NULL, *itime;
 
+	int i;
+
 	time(&t);
 	if (!bs->connlist)
 		return;
@@ -1928,6 +2172,22 @@ static void update_idle_time(struct buddy_show *bs)
 	if (blist_options & OPT_BLIST_SHOW_IDLETIME)
 		gtk_widget_show(bs->idle);
 
+	style = gtk_style_new();
+	gdk_font_unref(gtk_style_get_font(style));
+	gtk_style_set_font(style, gdk_font_ref(gtk_style_get_font(bs->label->style)));
+	for (i = 0; i < 5; i++)
+		style->fg[i] = bs->idle->style->fg[i];
+	if ((blist_options & OPT_BLIST_GREY_IDLERS) && (b->idle)) {
+		style->fg[GTK_STATE_NORMAL].red =
+		  (style->fg[GTK_STATE_NORMAL].red / 2) + (style->base[GTK_STATE_NORMAL].red / 2);
+		style->fg[GTK_STATE_NORMAL].green = 
+		  (style->fg[GTK_STATE_NORMAL].green / 2) + (style->base[GTK_STATE_NORMAL].green / 2);
+		style->fg[GTK_STATE_NORMAL].blue = 
+		  (style->fg[GTK_STATE_NORMAL].blue / 2) + (style->base[GTK_STATE_NORMAL].blue / 2);
+	}
+	gtk_widget_set_style(bs->label, style);
+	gtk_style_unref(style);
+
 	/* now we do the tooltip */
 	if (b->signon) {
 		char *stime = sec_to_text(t - b->signon +
@@ -2010,11 +2270,15 @@ void set_buddy(struct gaim_connection *gc, struct buddy *b)
 			gs = new_group_show(g->name);
 		if ((bs = find_buddy_show(gs, b->name)) == NULL)
 			bs = new_buddy_show(gs, b, (char **)login_icon_xpm);
-		if (!g_slist_find(bs->connlist, gc))
+		if (!g_slist_find(bs->connlist, gc)) {
 			bs->connlist = g_slist_append(bs->connlist, gc);
+			update_num_group(gs);
+		}
 		if (b->present == 1) {
 			if (bs->sound != 2)
-				play_sound(BUDDY_ARRIVE);
+				play_sound(SND_BUDDY_ARRIVE);
+			if (blist_options & OPT_BLIST_POPUP)
+				unhide_buddy_list();
 			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm,
 							  NULL, (char **)login_icon_xpm);
 			gtk_widget_hide(bs->pix);
@@ -2030,19 +2294,27 @@ void set_buddy(struct gaim_connection *gc, struct buddy *b)
 			if (bs->log_timer > 0)
 				gtk_timeout_remove(bs->log_timer);
 			bs->log_timer = gtk_timeout_add(10000, (GtkFunction)log_timeout, bs);
-			update_num_group(gs);
 			if ((bs->sound != 2) && (im_options & OPT_IM_LOGON)) {
 				struct conversation *c = find_conversation(b->name);
 				if (c) {
 					char tmp[1024];
-					g_snprintf(tmp, sizeof(tmp), _("%s logged in."), b->name);
-					write_to_conv(c, tmp, WFLAG_SYSTEM, NULL, time((time_t) NULL));
+					g_snprintf(tmp, sizeof(tmp), _("%s logged in."), b->show);
+					write_to_conv(c, tmp, WFLAG_SYSTEM, NULL, time(NULL), -1);
+				} else if (clistqueue && find_queue_total_by_name(b->name)) {
+					struct queued_message *qm = g_new0(struct queued_message, 1);
+					g_snprintf(qm->name, sizeof(qm->name), "%s", b->name);
+					qm->message = g_strdup_printf(_("%s logged in."), b->show);
+					qm->gc = gc;
+					qm->tm = time(NULL);
+					qm->flags = WFLAG_SYSTEM;
+					qm->len = -1;
+					message_queue = g_slist_append(message_queue, qm);
 				}
 			}
 			bs->sound = 2;
 		} else if (bs->log_timer == 0) {
 			if (gc->prpl->list_icon)
-				xpm = (*gc->prpl->list_icon)(b->uc);
+				xpm = gc->prpl->list_icon(b->uc);
 			if (xpm == NULL)
 				xpm = (char **)no_icon_xpm;
 			pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm, NULL, xpm);
@@ -2069,13 +2341,14 @@ void set_buddy(struct gaim_connection *gc, struct buddy *b)
 				   buddies that have already signed
 				   off */
 		if (bs->sound != 1)
-			play_sound(BUDDY_LEAVE);
-
+			play_sound(SND_BUDDY_LEAVE);
+		if (blist_options & OPT_BLIST_POPUP)
+			unhide_buddy_list();
 		bs->connlist = g_slist_remove(bs->connlist, gc);
+		update_num_group(gs);
 		if (bs->log_timer > 0)
 			gtk_timeout_remove(bs->log_timer);
 		bs->log_timer = gtk_timeout_add(10000, (GtkFunction)log_timeout, bs);
-		update_num_group(gs);
 		pm = gdk_pixmap_create_from_xpm_d(blist->window, &bm, NULL, logout_icon_xpm);
 		gtk_widget_hide(bs->pix);
 		gtk_pixmap_set(GTK_PIXMAP(bs->pix), pm, bm);
@@ -2090,8 +2363,17 @@ void set_buddy(struct gaim_connection *gc, struct buddy *b)
 			struct conversation *c = find_conversation(b->name);
 			if (c) {
 				char tmp[1024];
-				g_snprintf(tmp, sizeof(tmp), _("%s logged out."), b->name);
-				write_to_conv(c, tmp, WFLAG_SYSTEM, NULL, time((time_t) NULL));
+				g_snprintf(tmp, sizeof(tmp), _("%s logged out."), b->show);
+				write_to_conv(c, tmp, WFLAG_SYSTEM, NULL, time(NULL), -1);
+			} else if (clistqueue && find_queue_total_by_name(b->name)) {
+				struct queued_message *qm = g_new0(struct queued_message, 1);
+				g_snprintf(qm->name, sizeof(qm->name), "%s", b->name);
+				qm->message = g_strdup_printf(_("%s logged out."), b->show);
+				qm->gc = gc;
+				qm->tm = time(NULL);
+				qm->flags = WFLAG_SYSTEM;
+				qm->len = -1;
+				message_queue = g_slist_append(message_queue, qm);
 			}
 		}
 
@@ -2107,6 +2389,10 @@ static void move_blist_window(GtkWidget *w, GdkEventConfigure *e, void *dummy)
 	gdk_window_get_position(blist->window, &x, &y);
 	gdk_window_get_size(blist->window, &width, &height);
 
+#ifdef USE_APPLET
+	if (applet_buddy_show){
+#endif
+
 	if (e->send_event) {	/* Is a position event */
 		if (blist_pos.x != x || blist_pos.y != y)
 			save = 1;
@@ -2125,6 +2411,10 @@ static void move_blist_window(GtkWidget *w, GdkEventConfigure *e, void *dummy)
 	if (save)
 		save_prefs();
 
+#ifdef USE_APPLET
+	}
+#endif
+	
 }
 
 
@@ -2168,7 +2458,7 @@ GtkWidget *gaim_new_item(GtkWidget *menu, const char *str)
 	return menuitem;
 }
 
-GtkWidget *gaim_new_item_with_pixmap(GtkWidget *menu, const char *str, char **xpm, GtkSignalFunc sf,
+GtkWidget *gaim_new_item_with_pixmap(GtkWidget *menu, const char *str, char **xpm, GtkSignalFunc sf, gpointer data,
 				     guint accel_key, guint accel_mods, char *mod)
 {
 	GtkWidget *menuitem;
@@ -2183,7 +2473,7 @@ GtkWidget *gaim_new_item_with_pixmap(GtkWidget *menu, const char *str, char **xp
 		gtk_menu_append(GTK_MENU(menu), menuitem);
 	if (sf)
 		/* passing 1 is necessary so if we sign off closing the account editor doesn't exit */
-		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sf, (void *)1);
+		gtk_signal_connect(GTK_OBJECT(menuitem), "activate", sf, data);
 	gtk_widget_show(menuitem);
 
 	/* Create our container */
@@ -2278,7 +2568,16 @@ void build_imchat_box(gboolean on)
 	}
 }
 
-
+extern GtkWidget *debugbutton;
+void clicked_debug (GtkWidget *widg, gpointer pntr)
+{	
+	if (debugbutton)
+		gtk_button_clicked(GTK_BUTTON(debugbutton));
+	else {
+		misc_options ^= OPT_MISC_DEBUG;
+		show_debug();
+	}
+}
 
 void show_buddy_list()
 {
@@ -2308,7 +2607,7 @@ void show_buddy_list()
 	}
 
 #ifdef USE_APPLET
-	blist = gtk_window_new(GTK_WINDOW_DIALOG);
+	GAIM_DIALOG(blist);
 #else
 	blist = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 #endif
@@ -2334,28 +2633,30 @@ void show_buddy_list()
 	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menuitem);
 
 	gaim_new_item_with_pixmap(menu, _("Add A Buddy"), add_small_xpm,
-				  GTK_SIGNAL_FUNC(add_buddy_callback), 'b', GDK_CONTROL_MASK, "Ctl+B");
+				  GTK_SIGNAL_FUNC(add_buddy_callback), NULL,  'b', GDK_CONTROL_MASK, "Ctl+B");
 	gaim_new_item_with_pixmap(menu, _("Join A Chat"), pounce_small_xpm,
-				  GTK_SIGNAL_FUNC(chat_callback), 'c', GDK_CONTROL_MASK, "Ctl+C");
+				  GTK_SIGNAL_FUNC(chat_callback), NULL, 'c', GDK_CONTROL_MASK, "Ctl+C");
 	gaim_new_item_with_pixmap(menu, _("New Instant Message"), send_small_xpm,
-				  GTK_SIGNAL_FUNC(show_im_dialog), 'i', GDK_CONTROL_MASK, "Ctl+I");
+				  GTK_SIGNAL_FUNC(show_im_dialog), NULL, 'i', GDK_CONTROL_MASK, "Ctl+I");
+	gaim_new_item_with_pixmap(menu, _("Get User Info"), search_small_xpm,
+				  GTK_SIGNAL_FUNC(show_info_dialog), NULL, 'j', GDK_CONTROL_MASK, "Ctl+J");
 
 	gaim_separator(menu);
 
 	gaim_new_item_with_pixmap(menu, _("Import Buddy List"), import_small_xpm,
-				  GTK_SIGNAL_FUNC(import_callback), 0, 0, 0);
+				  GTK_SIGNAL_FUNC(import_callback), NULL, 0, 0, 0);
 	/*gaim_new_item_with_pixmap(menu, _("Export Buddy List"), export_small_xpm,
 	   GTK_SIGNAL_FUNC(show_export_dialog), 0, 0, 0); */
 	gaim_separator(menu);
-	gaim_new_item_with_pixmap(menu, _("Signoff"), logout_icon_xpm,
-				  GTK_SIGNAL_FUNC(signoff_all), 'd', GDK_CONTROL_MASK, "Ctl+D");
+	gaim_new_item_with_pixmap(menu, _("Signoff"), logout_menu_xpm,
+				  GTK_SIGNAL_FUNC(signoff_all), (void*)1, 'd', GDK_CONTROL_MASK, "Ctl+D");
 
 #ifndef USE_APPLET
 	gaim_new_item_with_pixmap(menu, _("Quit"), exit_small_xpm,
-				  GTK_SIGNAL_FUNC(do_quit), 'q', GDK_CONTROL_MASK, "Ctl+Q");
+				  GTK_SIGNAL_FUNC(do_quit), NULL, 'q', GDK_CONTROL_MASK, "Ctl+Q");
 #else
 	gaim_new_item_with_pixmap(menu, _("Close"), close_small_xpm,
-				  GTK_SIGNAL_FUNC(applet_destroy_buddy), 'x', GDK_CONTROL_MASK, "Ctl+X");
+				  GTK_SIGNAL_FUNC(applet_destroy_buddy), NULL, 'x', GDK_CONTROL_MASK, "Ctl+X");
 #endif
 
 	menu = gtk_menu_new();
@@ -2365,76 +2666,43 @@ void show_buddy_list()
 	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menuitem);
 
 	awaymenu = gtk_menu_new();
-	menuitem = gaim_new_item_with_pixmap(menu, _("Away"), away_small_xpm, NULL, 0, 0, 0);
+	menuitem = gaim_new_item_with_pixmap(menu, _("Away"), away_small_xpm, NULL, NULL, 0, 0, 0);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), awaymenu);
 	do_away_menu();
 
 	bpmenu = gtk_menu_new();
-	menuitem = gaim_new_item_with_pixmap(menu, _("Buddy Pounce"), pounce_small_xpm, NULL, 0, 0, 0);
+	menuitem = gaim_new_item_with_pixmap(menu, _("Buddy Pounce"), pounce_small_xpm, NULL, NULL, 0, 0, 0);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), bpmenu);
 	do_bp_menu();
 
 	gaim_separator(menu);
 
-#ifdef NO_MULTI
-	findmenu = gtk_menu_new();
-	gtk_widget_show(findmenu);
-	menuitem = gaim_new_item_with_pixmap(menu, _("Search for Buddy"), search_small_xpm,
-					     NULL, 0, 0, 0);
-	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), findmenu);
-	gtk_widget_show(menuitem);
-	menuitem = gtk_menu_item_new_with_label(_("by Email"));
-	gtk_menu_append(GTK_MENU(findmenu), menuitem);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(show_find_email), NULL);
-	gtk_widget_show(menuitem);
-	menuitem = gtk_menu_item_new_with_label(_("by Dir Info"));
-	gtk_menu_append(GTK_MENU(findmenu), menuitem);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(show_find_info),
-			   connections->data);
-	gtk_widget_show(menuitem);
-
-	setmenu = gtk_menu_new();
-	gtk_widget_show(setmenu);
-	menuitem = gaim_new_item_with_pixmap(menu, _("Settings"), prefs_small_xpm, NULL, 0, 0, 0);
-	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), setmenu);
-	menuitem = gtk_menu_item_new_with_label(_("User Info"));
-	gtk_menu_append(GTK_MENU(setmenu), menuitem);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(show_set_info), NULL);
-	gtk_widget_show(menuitem);
-	menuitem = gtk_menu_item_new_with_label(_("Directory Info"));
-	gtk_menu_append(GTK_MENU(setmenu), menuitem);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(show_set_dir), NULL);
-	gtk_widget_show(menuitem);
-	menuitem = gtk_menu_item_new_with_label(_("Change Password"));
-	gtk_menu_append(GTK_MENU(setmenu), menuitem);
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(show_change_passwd), NULL);
-	gtk_widget_show(menuitem);
-#else
+#ifndef NO_MULTI
 	gaim_new_item_with_pixmap(menu, _("Accounts"), add_small_xpm,
-				  GTK_SIGNAL_FUNC(account_editor), 'a', GDK_CONTROL_MASK, "Ctl+A");
+				  GTK_SIGNAL_FUNC(account_editor), NULL, 'a', GDK_CONTROL_MASK, "Ctl+A");
+#endif
 
 	protomenu = gtk_menu_new();
 	menuitem =
-	    gaim_new_item_with_pixmap(menu, _("Protocol Actions"), prefs_small_xpm, NULL, 0, 0, 0);
+	    gaim_new_item_with_pixmap(menu, _("Protocol Actions"), prefs_small_xpm, NULL, NULL, 0, 0, 0);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), protomenu);
 	do_proto_menu();
-#endif
 
 	gaim_new_item_with_pixmap(menu, _("Preferences"), prefs_small_xpm,
-				  GTK_SIGNAL_FUNC(show_prefs), 'p', GDK_CONTROL_MASK, "Ctl+P");
+				  GTK_SIGNAL_FUNC(show_prefs), NULL, 'p', GDK_CONTROL_MASK, "Ctl+P");
 	gaim_new_item_with_pixmap(menu, _("View System Log"), prefs_small_xpm,
-				  GTK_SIGNAL_FUNC(show_syslog), 0, 0, 0);
+				  GTK_SIGNAL_FUNC(show_syslog), NULL, 0, 0, 0);
 
 	gaim_separator(menu);
 
 #ifdef GAIM_PLUGINS
-	gaim_new_item_with_pixmap(menu, _("Plugins"), plugins_small_xpm, GTK_SIGNAL_FUNC(show_plugins),
+	gaim_new_item_with_pixmap(menu, _("Plugins"), plugins_small_xpm, GTK_SIGNAL_FUNC(show_plugins), NULL,
 				  0, 0, 0);
 #endif
 #ifdef USE_PERL
 	perlmenu = gtk_menu_new();
 	gtk_widget_show(perlmenu);
-	menuitem = gaim_new_item_with_pixmap(menu, _("Perl"), plugins_small_xpm, NULL, 0, 0, 0);
+	menuitem = gaim_new_item_with_pixmap(menu, _("Perl"), plugins_small_xpm, NULL, NULL, 0, 0, 0);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), perlmenu);
 	gtk_widget_show(menuitem);
 	menuitem = gtk_menu_item_new_with_label(_("Load Script"));
@@ -2458,22 +2726,23 @@ void show_buddy_list()
 	gtk_menu_item_right_justify(GTK_MENU_ITEM(menuitem));
 	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menuitem);
 
-	gaim_new_item_with_pixmap(menu, _("About Gaim"), about_small_xpm, show_about, GDK_F1, 0, NULL);
-
+	gaim_new_item_with_pixmap(menu, _("Online Help"), add_small_xpm, GTK_SIGNAL_FUNC(open_url), WEBSITE"documentation.php", GDK_F1, 0, NULL);
+	gaim_new_item_with_pixmap(menu, _("Debug Window"), search_small_xpm, GTK_SIGNAL_FUNC(clicked_debug), NULL, 0, 0, NULL);
+	gaim_new_item_with_pixmap(menu, _("About Gaim"), about_small_xpm, GTK_SIGNAL_FUNC(show_about), NULL, GDK_F1, GDK_CONTROL_MASK, NULL);
+	
+	
 	gtk_widget_show(menubar);
 
 	vbox = gtk_vbox_new(FALSE, 0);
 
 	notebook = gtk_notebook_new();
 
-
-
-
 	/* Do buddy list stuff */
 	/* FIXME: spacing on both panes is ad hoc */
 	buddypane = gtk_vbox_new(FALSE, 1);
 
 	buddies = gtk_tree_new();
+	gtk_tree_set_view_lines(GTK_TREE(buddies), FALSE);
 	sw = gtk_scrolled_window_new(NULL, NULL);
 
 	tips = gtk_tooltips_new();
@@ -2508,7 +2777,7 @@ void show_buddy_list()
 	}
 
 	edittree = gtk_ctree_new(1, 0);
-	gtk_ctree_set_line_style(GTK_CTREE(edittree), GTK_CTREE_LINES_SOLID);
+	gtk_ctree_set_line_style(GTK_CTREE(edittree), GTK_CTREE_LINES_SOLID);;
 	gtk_ctree_set_expander_style(GTK_CTREE(edittree), GTK_CTREE_EXPANDER_SQUARE);
 	gtk_clist_set_reorderable(GTK_CLIST(edittree), TRUE);
 	gtk_signal_connect(GTK_OBJECT(edittree), "button_press_event",
@@ -2562,6 +2831,9 @@ void show_buddy_list()
 	label = gtk_label_new(_("Edit Buddies"));
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), editpane, label);
 
+	if(blist_options & OPT_BLIST_BOTTOM_TAB)
+		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
+	
 	gtk_widget_show_all(notebook);
 
 	/* Pack things in the vbox */
