@@ -1,7 +1,8 @@
-/* $Id: libgg.c,v 1.2 2001/10/01 06:56:34 warmenhoven Exp $ */
+/* $Id: libgg.c,v 1.6 2001/12/05 09:48:56 warmenhoven Exp $ */
 
 /*
- *  (C) Copyright 2001 Wojtek Kaniewski <wojtekka@irc.pl>
+ *  (C) Copyright 2001 Wojtek Kaniewski <wojtekka@irc.pl>,
+ *                     Robert J. Wony <speedy@ziew.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -26,40 +27,39 @@
 #include <arpa/inet.h>
 #include <sys/ioctl.h>
 #include <sys/wait.h>
+#include <sys/time.h>
 #include <netdb.h>
 #include <errno.h>
-#include <string.h>
+#ifndef _AIX
+#  include <string.h>
+#endif
 #include <stdarg.h>
 #include <pwd.h>
+#include <time.h>
+#ifdef sun
+  #include <sys/filio.h>
+#endif
 #include <glib.h>
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+#  define WORDS_BIGENDIAN 1
+#endif
 #include "libgg.h"
+#include "config.h"
 
 int gg_debug_level = 0;
+int gg_http_use_proxy = 0;
+int gg_http_proxy_port = 0;
+char *gg_http_proxy_host = NULL;
 
-#ifdef GG_DEBUG
+#ifndef lint 
 
-/*
- * gg_debug_real()
- *
- * wyrzuca komunikat o danym poziomie, o ile uytkownik sobie tego yczy.
- *
- *  - level - poziom wiadomoci,
- *  - format... - tre wiadomoci (printf-alike.)
- *
- * niczego nie zwraca.
- */
-void gg_debug_real(int level, char *format, ...)
-{
-	va_list ap;
-	
-	if ((gg_debug_level & level)) {
-		va_start(ap, format);
-		vprintf(format, ap);
-		va_end(ap);
-	}
-}
+static char rcsid[]
+#ifdef __GNUC__
+__attribute__ ((unused))
+#endif
+= "$Id: libgg.c,v 1.6 2001/12/05 09:48:56 warmenhoven Exp $";
 
-#endif /* GG_DEBUG */
+#endif 
 
 /*
  * fix32() // funkcja wewntrzna
@@ -68,7 +68,7 @@ void gg_debug_real(int level, char *format, ...)
  */
 static inline unsigned long fix32(unsigned long x)
 {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#ifndef WORDS_BIGENDIAN
 	return x;
 #else
 	return (unsigned long)
@@ -86,7 +86,7 @@ static inline unsigned long fix32(unsigned long x)
  */
 static inline unsigned short fix16(unsigned short x)
 {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#ifndef WORDS_BIGENDIAN
 	return x;
 #else
 	return (unsigned short)
@@ -95,39 +95,6 @@ static inline unsigned short fix16(unsigned short x)
 #endif
 }
 
-/*
- * gg_alloc_sprintf() // funkcja wewntrzna
- *
- * robi dokadnie to samo, co sprintf(), tyle e alokuje sobie wczeniej
- * miejsce na dane. jak znam ycie, ze wzgldu na rnice midzy funkcjami
- * vsnprintf() na rnych platformach, nie bdzie dziaao ;)
- *
- *  - format, ... - parametry takie same jak w innych funkcjach *printf()
- *
- * zwraca zaalokowany buforek, ktry wypadaoby pniej zwolni, lub NULL
- * jeli nie udao si wykona zadania.
- */
-char *gg_alloc_sprintf(char *format, ...)
-{
-	va_list ap;
-	char *buf = NULL;
-	int size;
-
-	va_start(ap, format);
-
-	if ((size = vsnprintf(buf, 0, format, ap)) < 0)
-		return NULL;
-
-	if (!(buf = malloc(size + 1)))
-		return NULL;
-
-	vsnprintf(buf, size + 1, format, ap);
-
-	va_end(ap);
-
-	return buf;
-}
-
 /*
  * gg_resolve() // funkcja wewntrzna
  *
@@ -183,87 +150,6 @@ int gg_resolve(int *fd, int *pid, char *hostname)
 	return 0;
 }
 
-/*
- * gg_connect() // funkcja wewntrzna
- *
- * czy si z serwerem. pierwszy argument jest typu (void *), eby nie
- * musie niczego inkludowa w libgg.h i nie psu jaki gupich zalenoci
- * na dziwnych systemach.
- *
- *  - addr - adres serwera (struct in_addr *),
- *  - port - port serwera,
- *  - async - ma by asynchroniczne poczenie?
- *
- * zwraca poczonego socketa lub -1 w przypadku bdu. zobacz errno.
- */
-int gg_connect(void *addr, int port, int async)
-{
-	int sock, one = 1;
-	struct sockaddr_in sin;
-	struct in_addr *a = addr;
-
-	gg_debug(GG_DEBUG_FUNCTION, "** gg_connect(%s, %d, %d);\n", inet_ntoa(*a), port, async);
-	
-	if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
-		gg_debug(GG_DEBUG_MISC, "-- socket() failed. errno = %d (%s)\n", errno, strerror(errno));
-		return -1;
-	}
-
-	if (async) {
-		if (ioctl(sock, FIONBIO, &one) == -1) {
-			gg_debug(GG_DEBUG_MISC, "-- ioctl() failed. errno = %d (%s)\n", errno, strerror(errno));
-			return -1;
-		}
-	}
-
-	sin.sin_port = htons(port);
-	sin.sin_family = AF_INET;
-	sin.sin_addr.s_addr = a->s_addr;
-	
-	if (connect(sock, (struct sockaddr*) &sin, sizeof(sin)) == -1) {
-		gg_debug(GG_DEBUG_MISC, "-- connect() failed. errno = %d (%s)\n", errno, strerror(errno));
-		if (errno && (!async || errno != EINPROGRESS))
-			return -1;
-	}
-	
-	return sock;
-}
-
-/*
- * gg_read_line() // funkcja wewntrzna
- *
- * czyta jedn lini tekstu z socketa.
- *
- *  - sock - socket,
- *  - buf - wskanik bufora,
- *  - length - dugo bufora.
- *
- * olewa bdy. jeli na jaki trafi, potraktuje go jako koniec linii.
- */
-static void gg_read_line(int sock, char *buf, int length)
-{
-	int ret;
-
-	gg_debug(GG_DEBUG_FUNCTION, "** gg_read_line(...);\n");
-	
-	for (; length > 1; buf++, length--) {
-		do {
-			if ((ret = read(sock, buf, 1)) == -1 && errno != EINTR) {
-				*buf = 0;
-				return;
-			}
-		} while (ret == -1 && errno == EINTR);
-
-		if (*buf == '\n') {
-			buf++;
-			break;
-		}
-	}
-
-	*buf = 0;
-	return;
-}
-
 /*
  * gg_recv_packet() // funkcja wewntrzna
  *
@@ -354,7 +240,6 @@ static void *gg_recv_packet(struct gg_session *sess)
 
 	sess->recv_left = 0;
 
-#ifdef GG_DEBUG
 	if ((gg_debug_level & GG_DEBUG_DUMP)) {
 		int i;
 
@@ -363,7 +248,6 @@ static void *gg_recv_packet(struct gg_session *sess)
 			gg_debug(GG_DEBUG_DUMP, " %.2x", (unsigned char) buf[i]);
 		gg_debug(GG_DEBUG_DUMP, "\n");
 	}
-#endif
 
 	return buf;
 }
@@ -412,7 +296,6 @@ static int gg_send_packet(int sock, int type, void *packet, int length, void *pa
 	if (payload)
 		memcpy(tmp + sizeof(struct gg_header) + length, payload, payload_length);
 
-#ifdef GG_DEBUG
 	if ((gg_debug_level & GG_DEBUG_DUMP)) {
 		int i;
 
@@ -421,7 +304,6 @@ static int gg_send_packet(int sock, int type, void *packet, int length, void *pa
 			gg_debug(GG_DEBUG_DUMP, " %.2x", (unsigned char) tmp[i]);
 		gg_debug(GG_DEBUG_DUMP, "\n");
 	}
-#endif
 
 	plen = sizeof(struct gg_header) + length + payload_length;
 	
@@ -455,6 +337,8 @@ static int gg_send_packet(int sock, int type, void *packet, int length, void *pa
 struct gg_session *gg_login(uin_t uin, char *password, int async)
 {
 	struct gg_session *sess;
+	char *hostname;
+	int port;
 
 	gg_debug(GG_DEBUG_FUNCTION, "** gg_login(%u, \"...\", %d);\n", uin, async);
 
@@ -471,9 +355,21 @@ struct gg_session *gg_login(uin_t uin, char *password, int async)
 	sess->async = async;
 	sess->seq = 0;
 	sess->recv_left = 0;
+	sess->last_pong = 0;
+	sess->server_ip = 0;
+	sess->initial_status = 0;
+        sess->type = GG_SESSION_GG;
+	
+	if (gg_http_use_proxy) {
+		hostname = gg_http_proxy_host;
+		port = gg_http_proxy_port;
+	} else {
+		hostname = GG_APPMSG_HOST;
+		port = GG_APPMSG_PORT;
+	};
 
 	if (async) {
-		if (gg_resolve(&sess->fd, &sess->pid, GG_APPMSG_HOST)) {
+		if (gg_resolve(&sess->fd, &sess->pid, hostname)) {
 			gg_debug(GG_DEBUG_MISC, "-- resolving failed\n");
 			free(sess);
 			return NULL;
@@ -481,24 +377,24 @@ struct gg_session *gg_login(uin_t uin, char *password, int async)
 	} else {
 		struct in_addr a;
 
-		if ((a.s_addr = inet_addr(GG_APPMSG_HOST)) == INADDR_NONE) {
+		if ((a.s_addr = inet_addr(hostname)) == INADDR_NONE) {
 			struct hostent *he;
 	
-			if (!(he = gethostbyname(GG_APPMSG_HOST))) {
-				gg_debug(GG_DEBUG_MISC, "-- host %s not found\n", GG_APPMSG_HOST);
+			if (!(he = gethostbyname(hostname))) {
+				gg_debug(GG_DEBUG_MISC, "-- host %s not found\n", hostname);
 				free(sess);
 				return NULL;
 			} else
 				memcpy((char*) &a, he->h_addr, sizeof(a));
 		}
 
-		if (!(sess->fd = gg_connect(&a, GG_APPMSG_PORT, 0)) == -1) {
+		if (!(sess->fd = gg_connect(&a, port, 0)) == -1) {
 			gg_debug(GG_DEBUG_MISC, "-- connection failed\n");
 			free(sess);
 			return NULL;
 		}
 
-		sess->state = GG_STATE_CONNECTING_HTTP;
+		sess->state = GG_STATE_CONNECTING;
 
 		while (sess->state != GG_STATE_CONNECTED) {
 			struct gg_event *e;
@@ -510,6 +406,7 @@ struct gg_session *gg_login(uin_t uin, char *password, int async)
 			}
 
 			if (e->type == GG_EVENT_CONN_FAILED) {
+				errno = EACCES;
 				gg_debug(GG_DEBUG_MISC, "-- could not login\n");
 				gg_free_event(e);
 				free(sess);
@@ -835,6 +732,7 @@ static int gg_watch_fd_connected(struct gg_session *sess, struct gg_event *e)
 			e->event.msg.msgclass = fix32(r->msgclass);
 			e->event.msg.sender = fix32(r->sender);
 			e->event.msg.message = strdup((char*) r + sizeof(*r));
+			e->event.msg.time = fix32(r->time);
 		}
 	}
 
@@ -885,31 +783,17 @@ static int gg_watch_fd_connected(struct gg_session *sess, struct gg_event *e)
 		}
 	}
 
+	if (h->type == GG_PONG) {
+		gg_debug(GG_DEBUG_MISC, "-- received a pong\n");
+
+		sess->last_pong = time(NULL);
+	}
+
 	free(h);
 
 	return 0;
 }
 
-/*
- * gg_chomp() // funkcja wewntrzna
- *
- * ucina "\r\n" lub "\n" z koca linii.
- *
- *  - line - ofiara operacji plastycznej.
- *
- * niczego nie zwraca.
- */
-static void gg_chomp(char *line)
-{
-	if (!line || strlen(line) < 1)
-		return;
-
-	if (line[strlen(line) - 1] == '\n')
-		line[strlen(line) - 1] = 0;
-	if (line[strlen(line) - 1] == '\r')
-		line[strlen(line) - 1] = 0;
-}
-
 /*
  * gg_watch_fd()
  *
@@ -928,6 +812,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 {
 	struct gg_event *e;
 	int res = 0;
+	int port;
 
 	if (!sess) {
 		errno = EFAULT;
@@ -953,6 +838,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			if (read(sess->fd, &a, sizeof(a)) < sizeof(a) || a.s_addr == INADDR_NONE) {
 				gg_debug(GG_DEBUG_MISC, "-- resolving failed\n");				
 
+				errno = ENOENT;
 				e->type = GG_EVENT_CONN_FAILED;
 				e->event.failure = GG_FAILURE_RESOLVING;
 				sess->state = GG_STATE_IDLE;
@@ -961,86 +847,127 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 
 				break;
 			}
+			
+			sess->server_ip = a.s_addr;
 
 			close(sess->fd);
 
 			waitpid(sess->pid, NULL, 0);
 
 			gg_debug(GG_DEBUG_MISC, "-- resolved, now connecting\n");
+			
+			if (gg_http_use_proxy) {
+				port = gg_http_proxy_port;
+			} else {
+				port = GG_APPMSG_PORT;
+			};
 
-			if ((sess->fd = gg_connect(&a, GG_APPMSG_PORT, sess->async)) == -1) {
-				gg_debug(GG_DEBUG_MISC, "-- connection failed\n");
+			if ((sess->fd = gg_connect(&a, port, sess->async)) == -1) {
+				struct in_addr *addr = (struct in_addr*) &sess->server_ip;
+				
+				gg_debug(GG_DEBUG_MISC, "-- connection failed, trying direct connection\n");
 
-				e->type = GG_EVENT_CONN_FAILED;
-				e->event.failure = GG_FAILURE_CONNECTING;
-				sess->state = GG_STATE_IDLE;
-				break;
+				if ((sess->fd = gg_connect(addr, GG_DEFAULT_PORT, sess->async)) == -1) {
+				    gg_debug(GG_DEBUG_MISC, "-- connection failed, trying https connection\n");
+				    if ((sess->fd = gg_connect(&a, GG_HTTPS_PORT, sess->async)) == -1) {		
+					gg_debug(GG_DEBUG_MISC, "-- connect() failed. errno = %d (%s)\n", errno, strerror(errno));
+
+					e->type = GG_EVENT_CONN_FAILED;
+					e->event.failure = GG_FAILURE_CONNECTING;
+					sess->state = GG_STATE_IDLE;
+					break;
+				    }
+				}
+				sess->state = GG_STATE_CONNECTING_GG;
+				sess->check = GG_CHECK_WRITE;
+			} else {
+				sess->state = GG_STATE_CONNECTING;
+				sess->check = GG_CHECK_WRITE;
 			}
-
-			sess->state = GG_STATE_CONNECTING_HTTP;
-			sess->check = GG_CHECK_WRITE;
-
+				
 			break;
 		}
 
-		case GG_STATE_CONNECTING_HTTP:
+		case GG_STATE_CONNECTING:
 		{
 			char buf[1024];
 			int res, res_size = sizeof(res);
 
-			gg_debug(GG_DEBUG_MISC, "== GG_STATE_CONNECTING_HTTP\n");
+			gg_debug(GG_DEBUG_MISC, "== GG_STATE_CONNECTING\n");
 
 			if (sess->async && (getsockopt(sess->fd, SOL_SOCKET, SO_ERROR, &res, &res_size) || res)) {
-				gg_debug(GG_DEBUG_MISC, "-- http connection failed, errno = %d (%s)\n", res, strerror(res));
+				struct in_addr *addr = (struct in_addr*) &sess->server_ip;
+				gg_debug(GG_DEBUG_MISC, "-- http connection failed, errno = %d (%s), trying direct connection\n", res, strerror(res));
 
-				errno = res;
-				e->type = GG_EVENT_CONN_FAILED;
-				e->event.failure = GG_FAILURE_CONNECTING;
-				sess->state = GG_STATE_IDLE;
+				if ((sess->fd = gg_connect(addr, GG_DEFAULT_PORT, sess->async)) == -1) {
+				    gg_debug(GG_DEBUG_MISC, "-- connection failed, trying https connection\n");
+				    if ((sess->fd = gg_connect(addr, GG_HTTPS_PORT, sess->async)) == -1) {
+					gg_debug(GG_DEBUG_MISC, "-- connect() failed. errno = %d (%s)\n", errno, strerror(errno));
+
+					e->type = GG_EVENT_CONN_FAILED;
+					e->event.failure = GG_FAILURE_CONNECTING;
+					sess->state = GG_STATE_IDLE;
+					break;
+				    }
+				}
+
+				sess->state = GG_STATE_CONNECTING_GG;
+				sess->check = GG_CHECK_WRITE;
 				break;
 			}
-
+			
 			gg_debug(GG_DEBUG_MISC, "-- http connection succeded, sending query\n");
 
+			if (gg_http_use_proxy) {
+				snprintf(buf, sizeof(buf) - 1,
+					"GET http://" GG_APPMSG_HOST "/appsvc/appmsg.asp?fmnumber=%lu HTTP/1.0\r\n"
+					"Host: " GG_APPMSG_HOST "\r\n"
+					"User-Agent: " GG_HTTP_USERAGENT "\r\n"
+					"Pragma: no-cache\r\n"
+					"\r\n", sess->uin);
+			} else {
+				snprintf(buf, sizeof(buf) - 1,
+					"GET /appsvc/appmsg.asp?fmnumber=%lu HTTP/1.0\r\n"
+					"Host: " GG_APPMSG_HOST "\r\n"
+					"User-Agent: " GG_HTTP_USERAGENT "\r\n"
+					"Pragma: no-cache\r\n"
+					"\r\n", sess->uin);
+			};
 
-			snprintf(buf, sizeof(buf) - 1,
-				"GET /appsvc/appmsg.asp?fmnumber=%u HTTP/1.0\r\n"
-				"Host: " GG_APPMSG_HOST "\r\n"
-				"User-Agent: Mozilla/4.7 [en] (Win98; I)\r\n"
-				"Pragma: no-cache\r\n"
-				"\r\n", sess->uin);
-
+    			gg_debug(GG_DEBUG_MISC, "=> -----BEGIN-HTTP-QUERY-----\n%s\n=> -----END-HTTP-QUERY-----\n", buf);
+	 
 			if (write(sess->fd, buf, strlen(buf)) < strlen(buf)) {
 				gg_debug(GG_DEBUG_MISC, "-- sending query failed\n");
 
+				errno = EIO;
 				e->type = GG_EVENT_CONN_FAILED;
 				e->event.failure = GG_FAILURE_WRITING;
 				sess->state = GG_STATE_IDLE;
 				break;
 			}
 
-			sess->state = GG_STATE_WRITING_HTTP;
+			sess->state = GG_STATE_READING_DATA;
 			sess->check = GG_CHECK_READ;
 
 			break;
 		}
 
-		case GG_STATE_WRITING_HTTP:
+		case GG_STATE_READING_DATA:
 		{
 			char buf[1024], *tmp, *host;
 			int port = GG_DEFAULT_PORT;
 			struct in_addr a;
 
-			gg_debug(GG_DEBUG_MISC, "== GG_STATE_WRITING_HTTP\n");
+			gg_debug(GG_DEBUG_MISC, "== GG_STATE_READING_DATA\n");
 
 			gg_read_line(sess->fd, buf, sizeof(buf) - 1);
 			gg_chomp(buf);
 	
 			gg_debug(GG_DEBUG_TRAFFIC, "-- got http response (%s)\n", buf);
-	
 			if (strncmp(buf, "HTTP/1.", 7) || strncmp(buf + 9, "200", 3)) {
 				gg_debug(GG_DEBUG_MISC, "-- but that's not what we've expected\n");
 
+				errno = EINVAL;
 				e->type = GG_EVENT_CONN_FAILED;
 				e->event.failure = GG_FAILURE_INVALID;
 				sess->state = GG_STATE_IDLE;
@@ -1056,8 +983,9 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			close(sess->fd);
 	
 			gg_debug(GG_DEBUG_TRAFFIC, "-- received http data (%s)\n", buf);
-
+						
 			tmp = buf;
+			
 			while (*tmp && *tmp != ' ')
 				tmp++;
 			while (*tmp && *tmp == ' ')
@@ -1081,14 +1009,18 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			}
 
 			a.s_addr = inet_addr(host);
+			sess->server_ip = a.s_addr;
 
 			if ((sess->fd = gg_connect(&a, port, sess->async)) == -1) {
-				gg_debug(GG_DEBUG_MISC, "-- connect() failed. errno = %d (%s)\n", errno, strerror(errno));
+				gg_debug(GG_DEBUG_MISC, "-- connection failed, trying https connection\n");
+				if ((sess->fd = gg_connect(&a, GG_HTTPS_PORT, sess->async)) == -1) {
+				    gg_debug(GG_DEBUG_MISC, "-- connection failed, errno = %d (%s)\n", errno, strerror(errno));
 
-				e->type = GG_EVENT_CONN_FAILED;
-				e->event.failure = GG_FAILURE_CONNECTING;
-				sess->state = GG_STATE_IDLE;
-				break;
+				    e->type = GG_EVENT_CONN_FAILED;
+				    e->event.failure = GG_FAILURE_CONNECTING;
+				    sess->state = GG_STATE_IDLE;
+				    break;
+				}
 			}
 
 			sess->state = GG_STATE_CONNECTING_GG;
@@ -1104,24 +1036,28 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			gg_debug(GG_DEBUG_MISC, "== GG_STATE_CONNECTING_GG\n");
 
 			if (sess->async && (getsockopt(sess->fd, SOL_SOCKET, SO_ERROR, &res, &res_size) || res)) {
-				gg_debug(GG_DEBUG_MISC, "-- connection failed, errno = %d (%s)\n", errno, strerror(errno));
+				struct in_addr *addr = (struct in_addr*) &sess->server_ip;
 
-				errno = res;
-				e->type = GG_EVENT_CONN_FAILED;
-				e->event.failure = GG_FAILURE_CONNECTING;
-				sess->state = GG_STATE_IDLE;
-				break;
+				gg_debug(GG_DEBUG_MISC, "-- connection failed, trying https connection\n");
+				if ((sess->fd = gg_connect(addr, GG_HTTPS_PORT, sess->async)) == -1) {
+				    gg_debug(GG_DEBUG_MISC, "-- connection failed, errno = %d (%s)\n", errno, strerror(errno));
+				    
+				    e->type = GG_EVENT_CONN_FAILED;
+				    e->event.failure = GG_FAILURE_CONNECTING;
+				    sess->state = GG_STATE_IDLE;
+				    break;
+				}
 			}
 
 			gg_debug(GG_DEBUG_MISC, "-- connected\n");
 			
-			sess->state = GG_STATE_WAITING_FOR_KEY;
+			sess->state = GG_STATE_READING_KEY;
 			sess->check = GG_CHECK_READ;
 
 			break;
 		}
 
-		case GG_STATE_WAITING_FOR_KEY:
+		case GG_STATE_READING_KEY:
 		{
 			struct gg_header *h;			
 			struct gg_welcome *w;
@@ -1129,7 +1065,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			unsigned int hash;
 			char *password = sess->password;
 
-			gg_debug(GG_DEBUG_MISC, "== GG_STATE_WAITING_FOR_KEY\n");
+			gg_debug(GG_DEBUG_MISC, "== GG_STATE_READING_KEY\n");
 
 			if (!(h = gg_recv_packet(sess))) {
 				gg_debug(GG_DEBUG_MISC, "-- gg_recv_packet() failed. errno = %d (%s)\n", errno, strerror(errno));
@@ -1146,6 +1082,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 
 				free(h);
 				close(sess->fd);
+				errno = EINVAL;
 				e->type = GG_EVENT_CONN_FAILED;
 				e->event.failure = GG_FAILURE_INVALID;
 				sess->state = GG_STATE_IDLE;
@@ -1168,7 +1105,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 	
 			l.uin = fix32(sess->uin);
 			l.hash = fix32(hash);
-			l.status = fix32(GG_STATUS_AVAIL);
+			l.status = fix32(sess->initial_status ? sess->initial_status : GG_STATUS_AVAIL);
 			l.dunno = fix32(0x0b);
 			l.local_ip = 0;
 			l.local_port = 0;
@@ -1185,16 +1122,16 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 				break;
 			}
 	
-			sess->state = GG_STATE_SENDING_KEY;
+			sess->state = GG_STATE_READING_REPLY;
 
 			break;
 		}
 
-		case GG_STATE_SENDING_KEY:
+		case GG_STATE_READING_REPLY:
 		{
 			struct gg_header *h;
 
-			gg_debug(GG_DEBUG_MISC, "== GG_STATE_SENDING_KEY\n");
+			gg_debug(GG_DEBUG_MISC, "== GG_STATE_READING_REPLY\n");
 
 			if (!(h = gg_recv_packet(sess))) {
 				gg_debug(GG_DEBUG_MISC, "-- recv_packet failed\n");
@@ -1209,6 +1146,7 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 				gg_debug(GG_DEBUG_MISC, "-- login succeded\n");
 				e->type = GG_EVENT_CONN_SUCCESS;
 				sess->state = GG_STATE_CONNECTED;
+				free(h);
 				break;
 			}
 
@@ -1219,11 +1157,13 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			} else {
 				gg_debug(GG_DEBUG_MISC, "-- invalid packet\n");
 				e->event.failure = GG_FAILURE_INVALID;
+				errno = EINVAL;
 			}
 
 			e->type = GG_EVENT_CONN_FAILED;
 			sess->state = GG_STATE_IDLE;
 			close(sess->fd);
+			free(h);
 
 			break;
 		}
@@ -1254,4 +1194,12 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 	return e;
 }
 
-
+/*
+ * Local variables:
+ * c-indentation-style: k&r
+ * c-basic-offset: 8
+ * indent-tabs-mode: notnil
+ * End:
+ *
+ * vim: shiftwidth=8:
+ */
