@@ -13,6 +13,7 @@
 
 #include "pixmaps/msn_online.xpm"
 #include "pixmaps/msn_away.xpm"
+#include "pixmaps/msn_occ.xpm"
 
 /* MSN Smiley Faces */
 #include "pixmaps/protocols/msn/msn_angry.xpm"
@@ -53,7 +54,7 @@
 #include "pixmaps/protocols/msn/msn_thumbdown.xpm"
 #include "pixmaps/protocols/msn/msn_thumbup.xpm"
 #include "pixmaps/protocols/msn/msn_weird.xpm"
-#include "pixmaps/protocols/msn/msn_embaressed.xpm"
+#include "pixmaps/protocols/msn/msn_embarrassed.xpm"
 #include "pixmaps/protocols/msn/msn_angel.xpm"
 #include "pixmaps/protocols/msn/msn_film.xpm"
 #include "pixmaps/protocols/msn/msn_dog.xpm"
@@ -104,6 +105,13 @@ struct msn_data {
 	GSList *fl;
 	GSList *permit;
 	GSList *deny;
+
+	char *kv;
+	char *sid;
+	char *mspauth;
+	unsigned long sl;
+	char *passport;
+
 };
 
 struct msn_switchboard {
@@ -323,10 +331,12 @@ static char *handle_errcode(char *buf, gboolean show)
 		case 913:
 			g_snprintf(msg, sizeof(msg), "Not allowed when offline");
 			break;
-		case 920:
+	        case 920:
 			g_snprintf(msg, sizeof(msg), "Not accepting new users");
 			break;
-
+	        case 924:
+			g_snprintf(msg, sizeof(msg), "User unverified");
+			break;
 		default:
 			g_snprintf(msg, sizeof(msg), "Unknown Error Code");
 			break;
@@ -341,27 +351,37 @@ static char *handle_errcode(char *buf, gboolean show)
 static void handle_hotmail(struct gaim_connection *gc, char *data)
 {
 	char login_url[2048];
-	
-	snprintf(login_url, sizeof(login_url), "%s%s&passwd=%s", PASSPORT_URL, gc->username, gc->password);
+	char buf[MSN_BUF_LEN];
+	struct msn_data *md = gc->proto_data;
 
-	if (strstr(data, "Content-Type: text/x-msmsgsinitialemailnotification;")) {
-		char *x = strstr(data, "Inbox-Unread:");
-		if (!x) return;
-		x += strlen("Inbox-Unread: ");
-		connection_has_mail(gc, atoi(x), NULL, NULL, login_url);
-	} else if (strstr(data, "Content-Type: text/x-msmsgsemailnotification;")) {
-		char *from = strstr(data, "From:");
-		char *subject = strstr(data, "Subject:");
-		char *x;
-		if (!from || !subject) {
-			connection_has_mail(gc, 1, NULL, NULL, login_url);
+	if (!md->passport) {
+		g_snprintf(buf, sizeof(buf), "URL %d INBOX\r\n", ++md->trId);
+
+		if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 			return;
 		}
-		from += strlen("From: ");
-		x = strstr(from, "\r\n"); *x = 0;
-		subject += strlen("Subject: ");
-		x = strstr(subject, "\r\n"); *x = 0;
-		connection_has_mail(gc, -1, from, subject, login_url);
+	} else {
+		g_snprintf(login_url, sizeof(login_url), "%s", md->passport);
+
+		if (strstr(data, "Content-Type: text/x-msmsgsinitialemailnotification;")) {
+			char *x = strstr(data, "Inbox-Unread:");
+			if (!x) return;
+			x += strlen("Inbox-Unread: ");
+			connection_has_mail(gc, atoi(x), NULL, NULL, login_url);
+		} else if (strstr(data, "Content-Type: text/x-msmsgsemailnotification;")) {
+			char *from = strstr(data, "From:");
+			char *subject = strstr(data, "Subject:");
+			char *x;
+			if (!from || !subject) {
+				connection_has_mail(gc, 1, NULL, NULL, login_url);
+				return;
+			}
+			from += strlen("From: ");
+			x = strstr(from, "\r\n"); *x = 0;
+			subject += strlen("Subject: ");
+			x = strstr(subject, "\r\n"); *x = 0;
+			connection_has_mail(gc, -1, from, subject, login_url);
+		}
 	}
 }
 
@@ -552,9 +572,74 @@ static int msn_process_switch(struct msn_switchboard *ms, char *buf)
 	return 1;
 }
 
+static void msn_unescape(char *text) {
+	char *cpy = g_strdup(text);
+	char *cur = cpy;
+	int c = 0;
+
+
+	while (*cur) {
+		if (*cur == '%') {
+			if (sscanf (cur, "%%%x;", &c) == 1 && c != 0) {
+				*text = c;
+				cur = cur + 3;
+			} 
+		} else {
+			*text = *cur;
+			cur++;
+		}
+		text++;
+	}
+	*text = 0;
+	g_free(cpy);
+}
+
+static char *msn_parse_format(char *mime)
+{
+	char *cur;
+	GString *ret = g_string_new(NULL);
+	char colors[3];
+
+	cur = strstr(mime, "FN=");
+	if (cur && (*(cur = cur + 3) != ';')) {
+		ret = g_string_append(ret, "<FONT FACE=\"");
+		while (*cur && *cur != ';') {
+			ret = g_string_append_c(ret, *cur);
+			cur++;
+		}
+		ret = g_string_append(ret, "\">");
+	}
+	
+	cur = strstr(mime, "EF=");
+	if (cur && (*(cur = cur + 3) != ';')) {
+		while (*cur && *cur != ';') {
+			ret = g_string_append_c(ret, '<');
+			ret = g_string_append_c(ret, *cur);
+			ret = g_string_append_c(ret, '>');
+			cur++;
+		}
+	}
+	
+	cur = strstr(mime, "CO=");
+	if (cur && (*(cur = cur + 3) != ';')) {
+		if (sscanf (cur, "%x;", (int*)(&colors)) == 1) {
+			char tag[MSN_BUF_LEN];
+			g_snprintf(tag, sizeof(tag), "<FONT COLOR=\"#%02hhx%02hhx%02hhx\">", colors[0], colors[1], colors[2]);
+			ret = g_string_append(ret, tag);
+		}
+	}
+	
+	msn_unescape(ret->str);
+	cur = ret->str;
+	g_string_free(ret, FALSE);
+	return cur;
+}
+			
+	
 static void msn_process_switch_msg(struct msn_switchboard *ms, char *msg)
 {
-	char *content, *agent, *utf;
+	char *content, *agent, *format, *utf;
+	char *message = NULL;
 	int flags = 0;
 
 	agent = strstr(msg, "User-Agent: ");
@@ -562,6 +647,14 @@ static void msn_process_switch_msg(struct msn_switchboard *ms, char *msg)
 		if (!g_strncasecmp(agent, "User-Agent: Gaim", strlen("User-Agent: Gaim")))
 			flags |= IM_FLAG_GAIMUSER;
 	}
+
+	format = strstr(msg, "X-MMS-IM-Format: ");
+	if (format) {
+		format = msn_parse_format(format);
+	} else {
+		format = NULL;
+	}
+
 	content = strstr(msg, "Content-Type: ");
 	if (!content)
 		return;
@@ -571,8 +664,12 @@ static void msn_process_switch_msg(struct msn_switchboard *ms, char *msg)
 			serv_got_typing(ms->gc, ms->msguser, MSN_TYPING_RECV_TIMEOUT);
 			return;
 		} 
+
 	} else if (!g_strncasecmp(content, "Content-Type: text/plain",
 				  strlen("Content-Type: text/plain"))) {
+		
+	
+		
 		char *skiphead;
 		skiphead = strstr(msg, "\r\n\r\n");
 		if (!skiphead || !skiphead[4]) {
@@ -582,12 +679,24 @@ static void msn_process_switch_msg(struct msn_switchboard *ms, char *msg)
 		utf = utf8_to_str(skiphead);
 		strip_linefeed(utf);
 		
+		if (format) { 
+			int len = strlen(utf) + strlen(format) + 1;
+			message = g_malloc(len);
+			g_snprintf(message, len, "%s%s", format, utf);
+		} else {
+			message = utf;
+		}
+		
 		if (ms->chat)
-			serv_got_chat_in(ms->gc, ms->chat->id, ms->msguser, flags, utf, time(NULL));
+			serv_got_chat_in(ms->gc, ms->chat->id, ms->msguser, flags, message, time(NULL));
 		else
-			serv_got_im(ms->gc, ms->msguser, utf, flags, time(NULL), -1);
+			serv_got_im(ms->gc, ms->msguser, message, flags, time(NULL), -1);
 
-		g_free(utf);
+		g_free(message);
+		if (format) {
+			g_free(format);
+			g_free(utf);
+		}
 	}
 }
 
@@ -597,7 +706,7 @@ static void msn_switchboard_callback(gpointer data, gint source, GaimInputCondit
 	char buf[MSN_BUF_LEN];
 	int cont = 1;
 	int len;
-
+	
 	/* This is really stupid and I hate to put this here. */
 	if (ms->fd != source)
 		ms->fd = source;
@@ -749,12 +858,28 @@ static void msn_accept_add(gpointer w, struct msn_add_permit *map)
 		signoff(map->gc);
 		return;
 	}
+	map->gc->permit = g_slist_append(map->gc->permit, map->user);
 	build_allow_list(); /* er. right. we'll need to have a thing for this in CUI too */
 	show_got_added(map->gc, NULL, map->user, map->friend, NULL);
+	*(map->user) = 0;
 }
 
 static void msn_cancel_add(gpointer w, struct msn_add_permit *map)
 {
+	struct msn_data *md = map->gc->proto_data;
+	char buf[MSN_BUF_LEN];
+
+	if (*(map->user)) {
+		g_snprintf(buf, sizeof(buf), "ADD %d BL %s %s\r\n", ++md->trId, map->user, url_encode(map->friend));
+		if (msn_write(md->fd, buf, strlen(buf)) < 0) {
+			hide_login_progress(map->gc, "Write error");
+			signoff(map->gc);
+			return;
+		}
+		map->gc->deny = g_slist_append(map->gc->deny, map->user);
+		build_block_list();
+	}
+	
 	g_free(map->user);
 	g_free(map->friend);
 	g_free(map);
@@ -929,6 +1054,7 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		} else if (!g_strcasecmp(which, "BL") && pos) {
 			gc->deny = g_slist_append(gc->deny, g_strdup(who));
 		} else if (!g_strcasecmp(which, "RL")) {
+		    if (pos) {
 			while(perm) {
 				if(!g_strcasecmp(perm->data, who))
 					new = 0;
@@ -949,6 +1075,7 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		                g_snprintf(msg, sizeof(msg), "The user %s (%s) wants to add you to their buddy list",ap->user, url_decode(ap->friend));
 				do_ask_dialog(msg, ap, msn_accept_add, msn_cancel_add);
 			}
+		    }
 			
 			if (pos != tot) 
 				return 1; /* this isn't the last one in the RL, so return. */
@@ -1104,6 +1231,68 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		ms->sessid = g_strdup(sessid);
 		ms->auth = g_strdup(auth);
 		ms->gc = gc;
+	} else if (!g_strncasecmp(buf, "URL", 3)) {
+		char *tmp = buf;
+		FILE *fd;
+		md5_state_t st;
+		md5_byte_t di[16];
+		int i;
+		char buf2[64];
+		char sendbuf[64];
+		char hippy[2048];
+		char *rru;
+		char *passport;
+		char *filename;
+
+		GET_NEXT(tmp);
+		GET_NEXT(tmp);
+		rru = tmp;
+		GET_NEXT(tmp);
+		passport = tmp;
+		
+		snprintf(hippy, sizeof(hippy), "%s%d%s", md->mspauth, time(NULL) - md->sl, gc->password);
+
+		md5_init(&st);
+		md5_append(&st, (const md5_byte_t *)hippy, strlen(hippy));
+		md5_finish(&st, di);
+
+		bzero(sendbuf, sizeof(sendbuf));
+		for (i = 0; i < 16; i++) {
+			g_snprintf(buf2, sizeof(buf2), "%02x", di[i]);
+			strcat(sendbuf, buf2);
+		}
+
+		if (md->passport) {
+			unlink(md->passport);
+			g_free(md->passport);
+		}
+
+		fd = gaim_mkstemp(&(md->passport));
+		fprintf(fd, "<html>\n");
+		fprintf(fd, "<head>\n");
+		fprintf(fd, "<noscript>\n");
+		fprintf(fd, "<meta http-equiv=Refresh content=\"0; url=http://www.hotmail.com\">\n");
+		fprintf(fd, "</noscript>\n");
+		fprintf(fd, "</head>\n\n");
+		
+		fprintf(fd, "<body onload=\"document.pform.submit(); \">\n");
+		fprintf(fd, "<form name=\"pform\" action=\"%s\" method=\"POST\">\n\n", passport);
+		fprintf(fd, "<input type=\"hidden\" name=\"mode\" value=\"ttl\">\n");
+		fprintf(fd, "<input type=\"hidden\" name=\"login\" value=\"%s\">\n", gc->username);
+		fprintf(fd, "<input type=\"hidden\" name=\"username\" value=\"%s\">\n", gc->username);
+		fprintf(fd, "<input type=\"hidden\" name=\"sid\" value=\"%s\">\n", md->sid);
+		fprintf(fd, "<input type=\"hidden\" name=\"kv\" value=\"%s\">\n", md->kv);
+		fprintf(fd, "<input type=\"hidden\" name=\"id\" value=\"2\">\n");
+		fprintf(fd, "<input type=\"hidden\" name=\"sl\" value=\"%ld\">\n", time(NULL) - md->sl);
+		fprintf(fd, "<input type=\"hidden\" name=\"rru\" value=\"%s\">\n", rru);
+		fprintf(fd, "<input type=\"hidden\" name=\"auth\" value=\"%s\">\n", md->mspauth);
+		fprintf(fd, "<input type=\"hidden\" name=\"creds\" value=\"%s\">\n", sendbuf); // Digest me
+		fprintf(fd, "<input type=\"hidden\" name=\"svc\" value=\"mail\">\n");
+		fprintf(fd, "<input type=\"hidden\" name=\"js\" value=\"yes\">\n");
+		fprintf(fd, "</form></body>\n");
+		fprintf(fd, "</html>\n");
+		fclose(fd);
+
 	} else if (!g_strncasecmp(buf, "SYN", 3)) {
 	} else if (!g_strncasecmp(buf, "USR", 3)) {
 	} else if (!g_strncasecmp(buf, "XFR", 3)) {
@@ -1174,12 +1363,53 @@ static void msn_process_main_msg(struct gaim_connection *gc, char *msg)
 {
 	struct msn_data *md = gc->proto_data;
 	char *skiphead, *utf;
+	char *content;
+
+	content = strstr(msg, "Content-Type: ");
+
+	if ((content) && (!g_strncasecmp(content, "Content-Type: text/x-msmsgsprofile",
+				strlen("Content-Type: text/x-msmsgsprofile")))) {
+
+		char *kv,*sid,*mspauth;
+
+		kv = strstr(msg, "kv: ");
+		sid = strstr(msg, "sid: ");
+		mspauth = strstr(msg, "MSPAuth: ");
+
+		if (kv) {
+			char *tmp;
+
+			kv += strlen("kv: ");
+			tmp = strstr(kv, "\r\n"); *tmp = 0;
+			md->kv = g_strdup(kv);
+		}
+
+		if (sid) {
+			char *tmp;
+
+			sid += strlen("sid: ");
+			tmp = strstr(sid, "\r\n"); *tmp = 0;
+			md->sid = g_strdup(sid);
+		}
+
+		if (mspauth) {
+			char *tmp;
+
+			mspauth += strlen("MSPAuth: ");
+			tmp = strstr(mspauth, "\r\n"); *tmp = 0;
+			md->mspauth = g_strdup(mspauth);
+		}
+
+	}
+
+
 
 	if (!g_strcasecmp(md->msguser, "hotmail")) {
 		handle_hotmail(gc, msg);
 		return;
 	}
 
+
 	skiphead = strstr(msg, "\r\n\r\n");
 	if (!skiphead || !skiphead[4])
 		return;
@@ -1203,7 +1433,7 @@ static void msn_callback(gpointer data, gint source, GaimInputCondition cond)
 	len = read(md->fd, buf, sizeof(buf));
 
 	if (len <= 0) {
-		hide_login_progress(gc, "Error reading from server");
+		hide_login_progress_error(gc, "Error reading from server");
 		signoff(gc);
 		return;
 	}
@@ -1415,6 +1645,7 @@ static int msn_process_login(struct gaim_connection *gc, char *buf)
 		md->inpa = 0;
 		md->fd = 0;
 		md->fd = proxy_connect(host, port, msn_login_xfr_connect, gc);
+		md->sl = time(NULL);
 		if (md->fd < 0) {
 			hide_login_progress(gc, "Unable to transfer");
 			signoff(gc);
@@ -1546,6 +1777,10 @@ static void msn_close(struct gaim_connection *gc)
 	g_free(md->rxqueue);
 	if (md->msg)
 		g_free(md->msguser);
+	if (md->passport) {
+		unlink(md->passport);
+      		g_free(md->passport);
+	}
 	while (md->switches)
 		msn_kill_switch(md->switches->data);
 	while (md->fl) {
@@ -1564,7 +1799,6 @@ static int msn_send_typing(struct gaim_connection *gc, char *who, int typing) {
 	struct msn_switchboard *ms = msn_find_switch(gc, who);
 	char header[MSN_BUF_LEN] =   "MIME-Version: 1.0\r\n"
 				     "Content-Type: text/x-msmsgscontrol\r\n" 
-				     "User-Agent: Gaim/" VERSION "\r\n" 
 				     "TypingUser: ";
 	char buf [MSN_BUF_LEN];
 	if (!ms || !typing)
@@ -1754,7 +1988,12 @@ static char **msn_list_icon(int uc)
 {
 	if (uc == 0)
 		return msn_online_xpm;
-
+	
+	uc >>= 1;
+	
+	if (uc == 2 || uc == 6)
+		return msn_occ_xpm;
+	
 	return msn_away_xpm;
 }
 
@@ -1864,7 +2103,7 @@ static void msn_act_id(gpointer data, char *entry)
 	struct msn_data *md = gc->proto_data;
 	char buf[MSN_BUF_LEN];
 
-	if (strlen(url_encode(entry)) > 387) {
+	if (strlen(url_encode(entry)) >= BUDDY_ALIAS_MAXLEN) {
 		do_error_dialog("Friendly name too long.", "MSN Error");
 		return;
 	}
@@ -1958,58 +2197,62 @@ static void msn_set_permit_deny(struct gaim_connection *gc)
 	if (!md->permit && !md->deny)
 		return;
 
-	s = g_slist_nth(gc->permit, g_slist_length(md->permit));
-	while (s) {
-		char *who = s->data;
-		s = s->next;
-		if (!strchr(who, '@')) {
-			t = g_slist_append(t, who);
-			continue;
+	if (md->permit) {
+		s = g_slist_nth(gc->permit, g_slist_length(md->permit));
+		while (s) {
+			char *who = s->data;
+			s = s->next;
+			if (!strchr(who, '@')) {
+				t = g_slist_append(t, who);
+				continue;
+			}
+			g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, who, who);
+			if (msn_write(md->fd, buf, strlen(buf)) < 0) {
+				hide_login_progress(gc, "Write error");
+				signoff(gc);
+				return;
+			}
 		}
-		g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, who, who);
-		if (msn_write(md->fd, buf, strlen(buf)) < 0) {
-			hide_login_progress(gc, "Write error");
-			signoff(gc);
-			return;
+		while (t) {
+			char *who = t->data;
+			gc->permit = g_slist_remove(gc->permit, who);
+			g_free(who);
+			t = t->next;
 		}
-	}
-	while (t) {
-		char *who = t->data;
-		gc->permit = g_slist_remove(gc->permit, who);
-		g_free(who);
-		t = t->next;
-	}
-	if (t)
-		g_slist_free(t);
+		if (t)
+			g_slist_free(t);
 	t = NULL;
 	g_slist_free(md->permit);
 	md->permit = NULL;
-
-	s = g_slist_nth(gc->deny, g_slist_length(md->deny));
-	while (s) {
-		char *who = s->data;
-		s = s->next;
-		if (!strchr(who, '@')) {
-			t = g_slist_append(t, who);
-			continue;
-		}
-		g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, who, who);
-		if (msn_write(md->fd, buf, strlen(buf)) < 0) {
-			hide_login_progress(gc, "Write error");
-			signoff(gc);
-			return;
-		}
 	}
-	while (t) {
-		char *who = t->data;
-		gc->deny = g_slist_remove(gc->deny, who);
-		g_free(who);
-		t = t->next;
-	}
-	if (t)
-		g_slist_free(t);
+	
+	if (md->deny) {
+		s = g_slist_nth(gc->deny, g_slist_length(md->deny));
+		while (s) {
+			char *who = s->data;
+			s = s->next;
+			if (!strchr(who, '@')) {
+				t = g_slist_append(t, who);
+				continue;
+			}
+			g_snprintf(buf, sizeof(buf), "ADD %d BL %s %s\r\n", ++md->trId, who, who);
+			if (msn_write(md->fd, buf, strlen(buf)) < 0) {
+				hide_login_progress(gc, "Write error");
+				signoff(gc);
+				return;
+			}
+		}
+		while (t) {
+			char *who = t->data;
+			gc->deny = g_slist_remove(gc->deny, who);
+			g_free(who);
+			t = t->next;
+		}
+		if (t)
+			g_slist_free(t);
 	g_slist_free(md->deny);
 	md->deny = NULL;
+	}
 }
 
 static void msn_add_permit(struct gaim_connection *gc, char *who)
@@ -2180,8 +2423,9 @@ GSList *msn_smiley_list()
 	smilies = add_smiley(smilies, ";-)", msn_wink, 1);
 	smilies = add_smiley(smilies, ";)", msn_wink, 0);
 	smilies = add_smiley(smilies, ":S-", msn_wink, 0);
+	smilies = add_smiley(smilies, ":-$", msn_embarrassed, 1);
+	smilies = add_smiley(smilies, ":$", msn_embarrassed, 0);
 	
-
 	return smilies;
 }
 
