@@ -41,14 +41,44 @@
 #include "prpl.h"
 #include "gaim.h"
 #include "proxy.h"
+#include "md5.h"
 
 extern char *yahoo_crypt(char *, char *);
 
 #include "pixmaps/status-away.xpm"
 #include "pixmaps/status-here.xpm"
 #include "pixmaps/status-idle.xpm"
+#include "pixmaps/status-game.xpm"
 
-#undef YAHOO_DEBUG
+/* Yahoo Smilies go here */
+#include "pixmaps/protocols/yahoo/yahoo_alien.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_angel.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_angry.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_bigsmile.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_blush.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_bye.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_clown.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_cow.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_cowboy.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_cry.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_devil.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_flower.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_ghost.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_glasses.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_laughloud.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_love.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_mean.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_neutral.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_ooooh.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_question.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_sad.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_sleep.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_smiley.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_sunglas.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_tongue.xpm"
+#include "pixmaps/protocols/yahoo/yahoo_wink.xpm"
+
+#define YAHOO_DEBUG
 
 #define USEROPT_MAIL 0
 
@@ -92,7 +122,10 @@ enum yahoo_service { /* these are easier to see in hex */
 	YAHOO_SERVICE_GAMELOGOFF,
 	YAHOO_SERVICE_GAMEMSG = 0x2a,
 	YAHOO_SERVICE_FILETRANSFER = 0x46,
+	YAHOO_SERVICE_NOTIFY = 0x4B,
+	YAHOO_SERVICE_AUTHRESP = 0x54,
 	YAHOO_SERVICE_LIST = 0x55,
+	YAHOO_SERVICE_AUTH = 0x57,
 	YAHOO_SERVICE_ADDBUDDY = 0x83,
 	YAHOO_SERVICE_REMBUDDY = 0x84
 };
@@ -111,14 +144,17 @@ enum yahoo_status {
 	YAHOO_STATUS_INVISIBLE = 12,
 	YAHOO_STATUS_CUSTOM = 99,
 	YAHOO_STATUS_IDLE = 999,
-	YAHOO_STATUS_OFFLINE = 0x5a55aa56 /* don't ask */
+	YAHOO_STATUS_OFFLINE = 0x5a55aa56, /* don't ask */
+	YAHOO_STATUS_TYPING = 0x16
 };
+#define YAHOO_STATUS_GAME 0x2 /* Games don't fit into the regular status model */
 
 struct yahoo_data {
 	int fd;
 	guchar *rxqueue;
 	int rxlen;
 	GHashTable *hash;
+	GHashTable *games;
 	int current_status;
 	gboolean logged_in;
 };
@@ -342,11 +378,11 @@ static void yahoo_process_status(struct gaim_connection *gc, struct yahoo_packet
 {
 	struct yahoo_data *yd = gc->proto_data;
 	GSList *l = pkt->hash;
-	struct yahoo_packet *newpkt;
 	char *name = NULL;
 	int state = 0;
+	int gamestate = 0;
 	char *msg = NULL;
-
+	
 	while (l) {
 		struct yahoo_pair *pair = l->data;
 
@@ -358,14 +394,20 @@ static void yahoo_process_status(struct gaim_connection *gc, struct yahoo_packet
 				account_online(gc);
 				serv_finish_login(gc);
 				g_snprintf(gc->displayname, sizeof(gc->displayname), "%s", pair->value);
-				do_import(gc, NULL);
 				yd->logged_in = TRUE;
 
-				/* this requests the list. i have a feeling that this is very evil */
-				newpkt = yahoo_packet_new(YAHOO_SERVICE_LIST, YAHOO_STATUS_OFFLINE, 0);
-				yahoo_send_packet(yd, newpkt);
-				yahoo_packet_free(newpkt);
-			}
+				/* this requests the list. i have a feeling that this is very evil
+				 *
+				 * scs.yahoo.com sends you the list before this packet without  it being 
+				 * requested
+				 *
+				 * do_import(gc, NULL);
+				 * newpkt = yahoo_packet_new(YAHOO_SERVICE_LIST, YAHOO_STATUS_OFFLINE, 0);
+				 * yahoo_send_packet(yd, newpkt);
+				 * yahoo_packet_free(newpkt);
+				 */
+
+				}
 			break;
 		case 8: /* how many online buddies we have */
 			break;
@@ -388,10 +430,12 @@ static void yahoo_process_status(struct gaim_connection *gc, struct yahoo_packet
 				serv_got_update(gc, name, 0, 0, 0, 0, 0, 0);
 				break;
 			}
+			if (g_hash_table_lookup(yd->games, name))
+				gamestate = YAHOO_STATUS_GAME;
 			if (state == YAHOO_STATUS_AVAILABLE)
-				serv_got_update(gc, name, 1, 0, 0, 0, 0, 0);
-			else
-				serv_got_update(gc, name, 1, 0, 0, 0, (state << 1) | UC_UNAVAILABLE, 0);
+				serv_got_update(gc, name, 1, 0, 0, 0, gamestate, 0);
+			else 
+				serv_got_update(gc, name, 1, 0, 0, 0, (state << 2) | UC_UNAVAILABLE | gamestate, 0);
 			if (state == YAHOO_STATUS_CUSTOM) {
 				gpointer val = g_hash_table_lookup(yd->hash, name);
 				if (val) {
@@ -405,6 +449,9 @@ static void yahoo_process_status(struct gaim_connection *gc, struct yahoo_packet
 			break;
 		case 60: /* no clue */
 			 break;
+		case 16: /* Custom error message */
+			do_error_dialog(pair->value, "Gaim -- Yahoo! Error");
+			break;
 		default:
 			debug_printf("unknown status key %d\n", pair->key);
 			break;
@@ -431,6 +478,7 @@ static void yahoo_process_list(struct gaim_connection *gc, struct yahoo_packet *
 		if (pair->key != 87)
 			continue;
 
+		do_import(gc, NULL);
 		lines = g_strsplit(pair->value, "\n", -1);
 		for (tmp = lines; *tmp; tmp++) {
 			split = g_strsplit(*tmp, ":", 2);
@@ -456,13 +504,64 @@ static void yahoo_process_list(struct gaim_connection *gc, struct yahoo_packet *
 		do_export(gc);
 }
 
+static void yahoo_process_notify(struct gaim_connection *gc, struct yahoo_packet *pkt)
+{
+	char *msg = NULL;
+	char *from = NULL;
+	char *stat = NULL;
+	char *game = NULL;
+	GSList *l = pkt->hash;
+	struct yahoo_data *yd = (struct yahoo_data*) gc->proto_data;
+	while (l) {
+		struct yahoo_pair *pair = l->data;
+		if (pair->key == 4)
+			from = pair->value;
+		if (pair->key == 49)
+			msg = pair->value;
+		if (pair->key == 13)
+			stat = pair->value;
+		if (pair->key == 14)
+			game = pair->value;
+		l = l->next;
+	}
+	
+	if (!g_strncasecmp(msg, "TYPING", strlen("TYPING"))) {
+		if (*stat == '1')
+			serv_got_typing(gc, from, 0);
+		else
+			serv_got_typing_stopped(gc, from);
+	} else if (!g_strncasecmp(msg, "GAME", strlen("GAME"))) {
+		struct buddy *bud = find_buddy(gc, from);
+		void *free1=NULL, *free2=NULL;
+		if (!bud) 
+			debug_printf("%s is playing a game, and doesn't want you to know.\n");
+		if (*stat == '1') {
+			if (g_hash_table_lookup_extended (yd->games, from, free1, free2)) {
+				g_free(free1);
+				g_free(free2);
+			}
+			g_hash_table_insert (yd->games, g_strdup(from), g_strdup(game));
+			if (bud)
+				serv_got_update(gc, from, 1, 0, 0, 0, bud->uc | YAHOO_STATUS_GAME, 0);
+		} else {
+			if (g_hash_table_lookup_extended (yd->games, from, free1, free2)) {
+				g_free(free1);
+				g_free(free2);
+				g_hash_table_remove (yd->games, from);
+			}
+			if (bud)
+				serv_got_update(gc, from, 1, 0, 0, 0, bud->uc & ~YAHOO_STATUS_GAME, 0);
+		}
+	}
+}
+
 static void yahoo_process_message(struct gaim_connection *gc, struct yahoo_packet *pkt)
 {
 	char *msg = NULL;
 	char *from = NULL;
 	time_t tm = time(NULL);
 	GSList *l = pkt->hash;
-
+	
 	while (l) {
 		struct yahoo_pair *pair = l->data;
 		if (pair->key == 4)
@@ -474,7 +573,7 @@ static void yahoo_process_message(struct gaim_connection *gc, struct yahoo_packe
 		l = l->next;
 	}
 
-	if (pkt->status <= 1) {
+	if (pkt->status <= 1 || pkt->status == 5) {
 		char *m;
 		int i, j;
 		strip_linefeed(msg);
@@ -532,9 +631,9 @@ static void yahoo_process_contact(struct gaim_connection *gc, struct yahoo_packe
 		if (state == YAHOO_STATUS_AVAILABLE)
 			serv_got_update(gc, name, 1, 0, 0, 0, 0, 0);
 		else if (state == YAHOO_STATUS_IDLE)
-			serv_got_update(gc, name, 1, 0, 0, time(NULL) - 600, (state << 1), 0);
+			serv_got_update(gc, name, 1, 0, 0, time(NULL) - 600, (state << 2), 0);
 		else
-			serv_got_update(gc, name, 1, 0, 0, 0, (state << 1) | UC_UNAVAILABLE, 0);
+			serv_got_update(gc, name, 1, 0, 0, 0, (state << 2) | UC_UNAVAILABLE, 0);
 		if (state == YAHOO_STATUS_CUSTOM) {
 			gpointer val = g_hash_table_lookup(yd->hash, name);
 			if (val) {
@@ -576,6 +675,163 @@ static void yahoo_process_mail(struct gaim_connection *gc, struct yahoo_packet *
 	} else
 		connection_has_mail(gc, count, NULL, NULL, "http://mail.yahoo.com/");
 }
+/* This is the y64 alphabet... it's like base64, but has a . and a _ */
+char base64digits[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789._";
+
+/* This is taken from Sylpheed by Hiroyuki Yamamoto.  We have our own tobase64 function
+ * in util.c, but it has a bug I don't feel like finding right now ;) */
+void to_y64(unsigned char *out, const unsigned char *in, int inlen)
+     /* raw bytes in quasi-big-endian order to base 64 string (NUL-terminated) */
+{
+	for (; inlen >= 3; inlen -= 3)
+		{
+			*out++ = base64digits[in[0] >> 2];
+			*out++ = base64digits[((in[0] << 4) & 0x30) | (in[1] >> 4)];
+			*out++ = base64digits[((in[1] << 2) & 0x3c) | (in[2] >> 6)];
+			*out++ = base64digits[in[2] & 0x3f];
+			in += 3;
+		}
+	if (inlen > 0)
+		{
+			unsigned char fragment;
+
+			*out++ = base64digits[in[0] >> 2];
+			fragment = (in[0] << 4) & 0x30;
+			if (inlen > 1)
+				fragment |= in[1] >> 4;
+			*out++ = base64digits[fragment];
+			*out++ = (inlen < 2) ? '-' : base64digits[(in[1] << 2) & 0x3c];
+			*out++ = '-';
+		}
+	*out = '\0';
+}
+
+static void yahoo_process_auth(struct gaim_connection *gc, struct yahoo_packet *pkt)
+{
+	char *seed = NULL;
+	char *sn   = NULL;
+	GSList *l = pkt->hash;
+	struct yahoo_data *yd = gc->proto_data;
+	
+	while (l) {
+		struct yahoo_pair *pair = l->data;
+		if (pair->key == 94)
+			seed = pair->value;
+		if (pair->key == 1)
+			sn = pair->value;
+		l = l->next;
+	}
+	
+	if (seed) {
+		struct yahoo_packet *pack;
+	
+		/* So, Yahoo has stopped supporting its older clients in India, and undoubtedly
+		 * will soon do so in the rest of the world.
+		 * 
+		 * The new clients use this authentication method.  I warn you in advance, it's
+		 * bizzare, convoluted, inordinately complicated.  It's also no more secure than
+		 * crypt() was.  The only purpose this scheme could serve is to prevent third
+		 * part clients from connecting to their servers.
+		 *
+		 * Sorry, Yahoo.
+		 */
+		
+		md5_byte_t result[16];
+		md5_state_t ctx;
+		char *crypt_result;
+		char *password_hash = g_malloc(25);
+		char *crypt_hash = g_malloc(25);
+		char *hash_string_p = g_malloc(50 + strlen(sn));
+		char *hash_string_c = g_malloc(50 + strlen(sn));
+		
+		char checksum;
+		
+		int sv;
+		
+		char *result6 = g_malloc(25);
+		char *result96 = g_malloc(25);
+
+		sv = seed[15];
+		sv = sv % 8;
+
+		md5_init(&ctx);
+		md5_append(&ctx, gc->password, strlen(gc->password));
+		md5_finish(&ctx, result);
+		to_y64(password_hash, result, 16);
+		
+		md5_init(&ctx);
+		crypt_result = yahoo_crypt(gc->password, "$1$_2S43d5f$");  
+		md5_append(&ctx, crypt_result, strlen(crypt_result));
+		md5_finish(&ctx, result);
+		to_y64(crypt_hash, result, 16);
+
+		switch (sv) {
+		case 1:
+		case 6:
+			checksum = seed[seed[9] % 16];
+			g_snprintf(hash_string_p, strlen(sn) + 50,
+                                   "%c%s%s%s", checksum, gc->username, seed, password_hash);
+                        g_snprintf(hash_string_c, strlen(sn) + 50,
+				   "%c%s%s%s", checksum, gc->username, seed, crypt_hash);
+			break;
+		case 2:
+		case 7:
+			checksum = seed[seed[15] % 16];
+			g_snprintf(hash_string_p, strlen(sn) + 50,
+                                   "%c%s%s%s", checksum, seed, password_hash, gc->username);
+                        g_snprintf(hash_string_c, strlen(sn) + 50,
+                                   "%c%s%s%s", checksum, seed, crypt_hash, gc->username);
+			break;
+		case 3:
+			checksum = seed[seed[1] % 16];
+			g_snprintf(hash_string_p, strlen(sn) + 50,
+                                   "%c%s%s%s", checksum, gc->username, password_hash, seed);
+                        g_snprintf(hash_string_c, strlen(sn) + 50,
+                                   "%c%s%s%s", checksum, gc->username, crypt_hash, seed);
+			break;
+		case 4:
+			checksum = seed[seed[3] % 16];
+			g_snprintf(hash_string_p, strlen(sn) + 50,
+				   "%c%s%s%s", checksum, password_hash, seed, gc->username);
+			g_snprintf(hash_string_c, strlen(sn) + 50,
+				   "%c%s%s%s", checksum, crypt_hash, seed, gc->username);
+			break;
+		case 0:
+		case 5:
+			checksum = seed[seed[7] % 16];
+			g_snprintf(hash_string_p, strlen(sn) + 50,
+                                   "%c%s%s%s", checksum, password_hash, gc->username, seed);
+                        g_snprintf(hash_string_c, strlen(sn) + 50,
+				   "%c%s%s%s", checksum, crypt_hash, gc->username, seed);
+			break;
+		}
+		
+		md5_init(&ctx);  
+		md5_append(&ctx, hash_string_p, strlen(hash_string_p));
+		md5_finish(&ctx, result);
+		to_y64(result6, result, 16);
+
+		md5_init(&ctx);  
+		md5_append(&ctx, hash_string_c, strlen(hash_string_c));
+		md5_finish(&ctx, result);
+		to_y64(result96, result, 16);
+
+		pack = yahoo_packet_new(YAHOO_SERVICE_AUTHRESP, YAHOO_STATUS_AVAILABLE, 0);
+		yahoo_packet_hash(pack, 0, gc->username);
+		yahoo_packet_hash(pack, 6, result6);
+		yahoo_packet_hash(pack, 96, result96);
+		yahoo_packet_hash(pack, 1, gc->username);
+		
+		yahoo_send_packet(yd, pack);
+		
+		g_free(password_hash);
+		g_free(crypt_hash);
+		g_free(hash_string_p);
+		g_free(hash_string_c);
+
+		yahoo_packet_free(pack);
+	}
+}
 
 static void yahoo_packet_process(struct gaim_connection *gc, struct yahoo_packet *pkt)
 {
@@ -585,8 +841,13 @@ static void yahoo_packet_process(struct gaim_connection *gc, struct yahoo_packet
 	case YAHOO_SERVICE_LOGOFF:
 	case YAHOO_SERVICE_ISAWAY:
 	case YAHOO_SERVICE_ISBACK:
+	case YAHOO_SERVICE_GAMELOGON:
+	case YAHOO_SERVICE_GAMELOGOFF:
 		yahoo_process_status(gc, pkt);
 		break;
+	case YAHOO_SERVICE_NOTIFY:
+		yahoo_process_notify(gc, pkt);
+		break;
 	case YAHOO_SERVICE_MESSAGE:
 	case YAHOO_SERVICE_GAMEMSG:
 		yahoo_process_message(gc, pkt);
@@ -600,6 +861,9 @@ static void yahoo_packet_process(struct gaim_connection *gc, struct yahoo_packet
 	case YAHOO_SERVICE_LIST:
 		yahoo_process_list(gc, pkt);
 		break;
+	case YAHOO_SERVICE_AUTH:
+		yahoo_process_auth(gc, pkt);
+		break;
 	default:
 		debug_printf("unhandled service 0x%02x\n", pkt->service);
 		break;
@@ -616,7 +880,7 @@ static void yahoo_pending(gpointer data, gint source, GaimInputCondition cond)
 	len = read(yd->fd, buf, sizeof(buf));
 
 	if (len <= 0) {
-		hide_login_progress(gc, "Unable to read");
+		hide_login_progress_error(gc, "Unable to read");
 		signoff(gc);
 		return;
 	}
@@ -648,8 +912,8 @@ static void yahoo_pending(gpointer data, gint source, GaimInputCondition cond)
 		pkt = yahoo_packet_new(0, 0, 0);
 
 		pkt->service = yahoo_get16(yd->rxqueue + pos); pos += 2;
-		debug_printf("Yahoo Service: 0x%02x Status: %d\n", pkt->service, pkt->status);
 		pkt->status = yahoo_get32(yd->rxqueue + pos); pos += 4;
+		debug_printf("Yahoo Service: 0x%02x Status: %d\n", pkt->service, pkt->status);
 		pkt->id = yahoo_get32(yd->rxqueue + pos); pos += 4;
 
 		yahoo_packet_read(pkt, yd->rxqueue + pos, pktlen);
@@ -690,12 +954,9 @@ static void yahoo_got_connected(gpointer data, gint source, GaimInputCondition c
 	yd = gc->proto_data;
 	yd->fd = source;
 
-	pkt = yahoo_packet_new(YAHOO_SERVICE_LOGON, YAHOO_STATUS_AVAILABLE, 0);
+	pkt = yahoo_packet_new(YAHOO_SERVICE_AUTH, YAHOO_STATUS_AVAILABLE, 0);
 
-	yahoo_packet_hash(pkt, 0, gc->username);
 	yahoo_packet_hash(pkt, 1, gc->username);
-	yahoo_packet_hash(pkt, 6, yahoo_crypt(gc->password, "$1$_2S43d5f$"));
-
 	yahoo_send_packet(yd, pkt);
 
 	yahoo_packet_free(pkt);
@@ -711,8 +972,18 @@ static void yahoo_login(struct aim_user *user) {
 
 	yd->fd = -1;
 	yd->hash = g_hash_table_new(g_str_hash, g_str_equal);
+	yd->games = g_hash_table_new(g_str_hash, g_str_equal);
 
-	if (proxy_connect(user->proto_opt[USEROPT_PAGERHOST][0] ?
+
+	if (!g_strncasecmp(user->proto_opt[USEROPT_PAGERHOST], "cs.yahoo.com", strlen("cs.yahoo.com"))) {
+		/* Figured out the new auth method -- cs.yahoo.com likes to disconnect on buddy remove and add now */
+		debug_printf("Setting new Yahoo! server.\n");
+		g_snprintf(user->proto_opt[USEROPT_PAGERHOST], strlen("scs.yahoo.com") + 1, "scs.yahoo.com");
+		save_prefs();
+	}
+	
+    
+       	if (proxy_connect(user->proto_opt[USEROPT_PAGERHOST][0] ?
 				user->proto_opt[USEROPT_PAGERHOST] : YAHOO_PAGER_HOST,
 			   user->proto_opt[USEROPT_PAGERPORT][0] ?
 				atoi(user->proto_opt[USEROPT_PAGERPORT]) : YAHOO_PAGER_PORT,
@@ -735,6 +1006,8 @@ static void yahoo_close(struct gaim_connection *gc) {
 	struct yahoo_data *yd = (struct yahoo_data *)gc->proto_data;
 	g_hash_table_foreach_remove(yd->hash, yahoo_destroy_hash, NULL);
 	g_hash_table_destroy(yd->hash);
+	g_hash_table_foreach_remove(yd->games, yahoo_destroy_hash, NULL);
+	g_hash_table_destroy(yd->games);
 	if (yd->fd >= 0)
 		close(yd->fd);
 	if (yd->rxqueue)
@@ -747,11 +1020,13 @@ static void yahoo_close(struct gaim_connection *gc) {
 
 static char **yahoo_list_icon(int uc)
 {
-	if ((uc >> 1) == YAHOO_STATUS_IDLE)
+	if ((uc >> 2) == YAHOO_STATUS_IDLE)
 		return status_idle_xpm;
-	else if (uc == 0)
-		return status_here_xpm;
-	return status_away_xpm;
+	else if (uc & UC_UNAVAILABLE)
+		return status_away_xpm;
+	else if (uc & YAHOO_STATUS_GAME)
+		return status_game_xpm;
+	return status_here_xpm;
 }
 
 static char *yahoo_get_status_string(enum yahoo_status a)
@@ -782,6 +1057,22 @@ static char *yahoo_get_status_string(enum yahoo_status a)
 	}
 }
 
+static void yahoo_game(struct gaim_connection *gc, char *name) {
+	struct yahoo_data *yd = (struct yahoo_data *)gc->proto_data;
+	char *game = g_hash_table_lookup(yd->games, name);
+	char *t;
+	char url[256];
+
+	if (!game)
+		return;
+	t = game = g_strdup(strstr(game, "ante?room="));
+	while (*t != '\t')
+		t++;
+	*t = 0;
+	g_snprintf(url, sizeof url, "http://games.yahoo.com/games/%s", game);
+	open_url(NULL, url);
+	g_free(game);
+}
 static GList *yahoo_buddy_menu(struct gaim_connection *gc, char *who)
 {
 	GList *m = NULL;
@@ -790,22 +1081,45 @@ static GList *yahoo_buddy_menu(struct gaim_connection *gc, char *who)
 	struct buddy *b = find_buddy(gc, who); /* this should never be null. if it is,
 						  segfault and get the bug report. */
 	static char buf[1024];
-
-	if (!(b->uc & UC_UNAVAILABLE))
-		return NULL;
-    if (b->uc >> 1 == YAHOO_STATUS_IDLE)
-        return NULL;
-
-	pbm = g_new0(struct proto_buddy_menu, 1);
-	if ((b->uc >> 1) != YAHOO_STATUS_CUSTOM)
-		g_snprintf(buf, sizeof buf, "Status: %s", yahoo_get_status_string(b->uc >> 1));
-	else
-		g_snprintf(buf, sizeof buf, "Custom Status: %s",
-			   (char *)g_hash_table_lookup(yd->hash, b->name));
-	pbm->label = buf;
-	pbm->callback = NULL;
-	pbm->gc = gc;
-	m = g_list_append(m, pbm);
+	static char buf2[1024];
+	
+	if (b->uc & UC_UNAVAILABLE && b->uc >> 2 != YAHOO_STATUS_IDLE) {
+		pbm = g_new0(struct proto_buddy_menu, 1);
+		if ((b->uc >> 2) != YAHOO_STATUS_CUSTOM)
+			g_snprintf(buf, sizeof buf, 
+				   "Status: %s", yahoo_get_status_string(b->uc >> 2));
+		else
+			g_snprintf(buf, sizeof buf, "Custom Status: %s",
+				   (char *)g_hash_table_lookup(yd->hash, b->name));
+		pbm->label = buf;
+		pbm->callback = NULL;
+		pbm->gc = gc;
+		m = g_list_append(m, pbm);
+	}
+	
+	if (b->uc | YAHOO_STATUS_GAME) {
+		char *game = g_hash_table_lookup(yd->games, b->name);
+		char *room;
+		if (!game)
+			return m;
+		if (game) {
+			char *t;
+			pbm = g_new0(struct proto_buddy_menu, 1);
+			if (!(room = strstr(game, "&follow="))) /* skip ahead to the url */
+				return NULL;
+			while (*room && *room != '\t')          /* skip to the tab */
+				room++;
+			t = room++;                             /* room as now at the name */
+			while (*t != '\n')         
+				t++;                            /* replace the \n with a space */
+			*t = ' ';
+			g_snprintf(buf2, sizeof buf2, "%s", room);
+			pbm->label = buf2;
+			pbm->callback = yahoo_game;
+			pbm->gc = gc;
+			m = g_list_append(m, pbm);
+		}
+	}
 
 	return m;
 }
@@ -858,7 +1172,7 @@ static GList *yahoo_actions() {
 	return m;
 }
 
-static int yahoo_send_im(struct gaim_connection *gc, char *who, char *what, int flags)
+static int yahoo_send_im(struct gaim_connection *gc, char *who, char *what, int len, int flags)
 {
 	struct yahoo_data *yd = gc->proto_data;
 	struct yahoo_packet *pkt = yahoo_packet_new(YAHOO_SERVICE_MESSAGE, YAHOO_STATUS_OFFLINE, 0);
@@ -874,6 +1188,24 @@ static int yahoo_send_im(struct gaim_connection *gc, char *who, char *what, int
 	return 1;
 }
 
+int yahoo_send_typing(struct gaim_connection *gc, char *who, int typ)
+{
+	struct yahoo_data *yd = gc->proto_data;
+	struct yahoo_packet *pkt = yahoo_packet_new(YAHOO_SERVICE_NOTIFY, YAHOO_STATUS_TYPING, 0);
+	yahoo_packet_hash(pkt, 49, "TYPING");
+	yahoo_packet_hash(pkt, 1, gc->displayname);
+	yahoo_packet_hash(pkt, 14, " ");
+	yahoo_packet_hash(pkt, 13, typ ? "1" : "0");
+	yahoo_packet_hash(pkt, 5, who);
+	yahoo_packet_hash(pkt, 1002, "1");
+
+	yahoo_send_packet(yd, pkt);
+
+	yahoo_packet_free(pkt);
+
+	return 0;
+}
+
 static void yahoo_set_away(struct gaim_connection *gc, char *state, char *msg)
 {
 	struct yahoo_data *yd = (struct yahoo_data *)gc->proto_data;
@@ -1025,6 +1357,76 @@ static void yahoo_remove_buddy(struct gaim_connection *gc, char *who, char *grou
 	yahoo_packet_free(pkt);
 }
 
+
+GSList *yahoo_smiley_list() 
+{ 
+	GSList *smilies = NULL;
+
+	smilies = add_smiley(smilies, "=:)", yahoo_alien, 1);
+	smilies = add_smiley(smilies, "=:-)", yahoo_alien, 0);
+	smilies = add_smiley(smilies, "o:)", yahoo_angel, 0);
+	smilies = add_smiley(smilies, "o:-)", yahoo_angel, 0);
+	smilies = add_smiley(smilies, "0:)", yahoo_angel, 0);
+	smilies = add_smiley(smilies, "0:-)", yahoo_angel, 0);
+	smilies = add_smiley(smilies, "X-(", yahoo_angry, 1);
+	smilies = add_smiley(smilies, "X(", yahoo_angry, 0);
+	smilies = add_smiley(smilies, "x-(", yahoo_angry, 0);
+	smilies = add_smiley(smilies, "x(", yahoo_angry, 0);
+	smilies = add_smiley(smilies, ":D", yahoo_bigsmile, 1);
+	smilies = add_smiley(smilies, ":-D", yahoo_bigsmile, 0);
+	smilies = add_smiley(smilies, ":\">", yahoo_blush, 1);
+	smilies = add_smiley(smilies, "=;", yahoo_bye, 1);
+	smilies = add_smiley(smilies, ":o)", yahoo_clown, 1);
+	smilies = add_smiley(smilies, ":0)", yahoo_clown, 0);
+	smilies = add_smiley(smilies, ":O)", yahoo_clown, 0);
+	smilies = add_smiley(smilies, "<@:)", yahoo_clown, 0);
+	smilies = add_smiley(smilies, "3:-0", yahoo_cow, 1);
+	smilies = add_smiley(smilies, "3:-o", yahoo_cow, 0);
+	smilies = add_smiley(smilies, "3:-O", yahoo_cow, 0);
+	smilies = add_smiley(smilies, "3:O", yahoo_cow, 0);
+	smilies = add_smiley(smilies, "<):)", yahoo_cowboy, 1);
+	smilies = add_smiley(smilies, ":((", yahoo_cry, 1);
+	smilies = add_smiley(smilies, ":-((", yahoo_cry, 0);
+	smilies = add_smiley(smilies, ">:)", yahoo_devil, 1);
+	smilies = add_smiley(smilies, "@};-", yahoo_flower, 1);
+	smilies = add_smiley(smilies, "8-X", yahoo_ghost, 1);
+	smilies = add_smiley(smilies, ":B", yahoo_glasses, 1);
+	smilies = add_smiley(smilies, ":-B", yahoo_glasses, 0);
+	smilies = add_smiley(smilies, ":))", yahoo_laughloud, 1);
+	smilies = add_smiley(smilies, ":-))", yahoo_laughloud, 0);
+	smilies = add_smiley(smilies, ":x", yahoo_love, 1);
+	smilies = add_smiley(smilies, ":-x", yahoo_love, 0);
+	smilies = add_smiley(smilies, ":X", yahoo_love, 0);
+	smilies = add_smiley(smilies, ":-X", yahoo_love, 0);
+	smilies = add_smiley(smilies, ":>", yahoo_mean, 1);
+	smilies = add_smiley(smilies, ":->", yahoo_mean, 0);
+	smilies = add_smiley(smilies, ":|", yahoo_neutral, 1);
+	smilies = add_smiley(smilies, ":-|", yahoo_neutral, 0);
+	smilies = add_smiley(smilies, ":O", yahoo_ooooh, 1);
+	smilies = add_smiley(smilies, ":-O", yahoo_ooooh, 0);
+	smilies = add_smiley(smilies, ":-\\", yahoo_question, 1);
+	smilies = add_smiley(smilies, ":-/", yahoo_question, 0);
+	smilies = add_smiley(smilies, ":(", yahoo_sad, 1);
+	smilies = add_smiley(smilies, ":-(", yahoo_sad, 0);
+	smilies = add_smiley(smilies, "I-)", yahoo_sleep, 1);
+	smilies = add_smiley(smilies, "|-)", yahoo_sleep, 0);
+	smilies = add_smiley(smilies, "I-|", yahoo_sleep, 0);
+	smilies = add_smiley(smilies, ":)", yahoo_smiley, 1);
+	smilies = add_smiley(smilies, ":-)", yahoo_smiley, 0);
+	smilies = add_smiley(smilies, "(:", yahoo_smiley, 0);
+	smilies = add_smiley(smilies, "(-:", yahoo_smiley, 0);
+	smilies = add_smiley(smilies, "B-)", yahoo_sunglas, 1);
+	smilies = add_smiley(smilies, ":-p", yahoo_tongue, 1);
+	smilies = add_smiley(smilies, ":p", yahoo_tongue, 0);
+	smilies = add_smiley(smilies, ":P", yahoo_tongue, 0);
+	smilies = add_smiley(smilies, ":-P", yahoo_tongue, 0);
+	smilies = add_smiley(smilies, ";)", yahoo_wink, 1);
+	smilies = add_smiley(smilies, ";-)", yahoo_wink, 0);
+
+
+	return smilies;
+}
+
 static struct prpl *my_protocol = NULL;
 
 void yahoo_init(struct prpl *ret) {
@@ -1045,6 +1447,8 @@ void yahoo_init(struct prpl *ret) {
 	ret->keepalive = yahoo_keepalive;
 	ret->add_buddy = yahoo_add_buddy;
 	ret->remove_buddy = yahoo_remove_buddy;
+	ret->send_typing = yahoo_send_typing;
+	ret->smiley_list = yahoo_smiley_list;
 
 	my_protocol = ret;
 }
