@@ -349,7 +349,7 @@ static void handle_message(ZNotice_t notice, struct sockaddr_in from)
 					g_string_sprintfa(str, "<br>At %s since %s", locs.host,
 									locs.time);
 				}
-				g_show_info_text(str->str, NULL);
+				g_show_info_text(NULL, NULL, 2, str->str, NULL);
 				g_string_free(str, TRUE);
 			} else
 				serv_got_update(zgc, b->name, nlocs, 0, 0, 0, 0, 0);
@@ -358,6 +358,7 @@ static void handle_message(ZNotice_t notice, struct sockaddr_in from)
 		}
 	} else {
 		char *buf, *buf2;
+		char *send_inst;
 		char *ptr = notice.z_message + strlen(notice.z_message) + 1;
 		int len = notice.z_message_len - (ptr - notice.z_message);
 		int away;
@@ -373,7 +374,7 @@ static void handle_message(ZNotice_t notice, struct sockaddr_in from)
 					away = TRUE;
 				else
 					away = FALSE;
-				serv_got_im(zgc, notice.z_sender, buf2, 0, time((time_t)NULL));
+				serv_got_im(zgc, notice.z_sender, buf2, 0, time(NULL), -1);
 			} else {
 				zephyr_triple *zt1, *zt2;
 				zt1 = new_triple(notice.z_class, notice.z_class_inst,
@@ -386,8 +387,11 @@ static void handle_message(ZNotice_t notice, struct sockaddr_in from)
 						zt2->open = TRUE;
 						serv_got_joined_chat(zgc, zt2->id, zt2->name);
 					}
-					serv_got_chat_in(zgc, zt2->id, notice.z_sender, FALSE,
-								buf2, time((time_t)NULL));
+					send_inst = g_strdup_printf("%s %s", notice.z_sender,
+								    notice.z_class_inst);
+					serv_got_chat_in(zgc, zt2->id, send_inst, FALSE,
+								buf2, time(NULL));
+					g_free(send_inst);
 				}
 				free_triple(zt1);
 			}
@@ -495,17 +499,37 @@ static void process_zsubs()
 			if (buff[0]) {
 				triple = g_strsplit(buff, ",", 3);
 				if (triple[0] && triple[1] && triple[2]) {
+					char *tmp = g_strdup_printf("%s@%s", g_getenv("USER"),
+								    ZGetRealm());
+					char *atptr;
 					sub.zsub_class = triple[0];
 					sub.zsub_classinst = triple[1];
 					if (!g_strcasecmp(triple[2], "%me%")) {
 						recip = g_strdup_printf("%s@%s", g_getenv("USER"),
 										ZGetRealm());
 					} else if (!g_strcasecmp(triple[2], "*")) {
-						/* wildcard */
-						recip = g_strdup_printf("@%s", ZGetRealm());
+						/* wildcard
+						 * form of class,instance,* */
+						recip = g_malloc0(1);
+					} else if (!g_strcasecmp(triple[2], tmp)) {
+						/* form of class,instance,aatharuv@ATHENA.MIT.EDU */
+						recip = g_strdup(triple[2]);
+					} else if ((atptr = strchr(triple[2], '@')) != NULL) {
+						/* form of class,instance,*@ANDREW.CMU.EDU
+						 * class,instance,@ANDREW.CMU.EDU
+						 * If realm is local realm, blank recipient, else
+						 * @REALM-NAME
+						 */
+						char *realmat = g_strdup_printf("@%s", ZGetRealm());
+						if (!g_strcasecmp(atptr, realmat))
+							recip = g_malloc0(1);
+						else
+							recip = g_strdup(atptr);
+						g_free(realmat);
 					} else {
 						recip = g_strdup(triple[2]);
 					}
+					g_free(tmp);
 					sub.zsub_recipient = recip;
 					if (ZSubscribeTo(&sub, 1, 0) != ZERR_NONE) {
 						debug_printf("Zephyr: Couldn't subscribe to %s, %s, "
@@ -574,7 +598,7 @@ static void zephyr_login(struct aim_user *user)
 	process_zsubs();
 
 	nottimer = g_timeout_add(100, check_notify, NULL);
-	loctimer = g_timeout_add(2000, check_loc, NULL);
+	loctimer = g_timeout_add(20000, check_loc, NULL);
 }
 
 static void write_zsubs()
@@ -670,7 +694,7 @@ static void zephyr_close(struct gaim_connection *gc)
 }
 
 static void zephyr_add_buddy(struct gaim_connection *gc, char *buddy) { }
-static void zephyr_remove_buddy(struct gaim_connection *gc, char *buddy) { }
+static void zephyr_remove_buddy(struct gaim_connection *gc, char *buddy, char *group) { }
 
 static int zephyr_chat_send(struct gaim_connection *gc, int id, char *im)
 {
@@ -712,7 +736,7 @@ static int zephyr_chat_send(struct gaim_connection *gc, int id, char *im)
 	return 0;
 }
 
-static int zephyr_send_im(struct gaim_connection *gc, char *who, char *im, int flags) {
+static int zephyr_send_im(struct gaim_connection *gc, char *who, char *im, int len, int flags) {
 	ZNotice_t notice;
 	char *buf;
 	const char *sig;
@@ -796,7 +820,7 @@ static void zephyr_set_away(struct gaim_connection *gc, char *state, char *msg)
 		gc->away = g_strdup(msg);
 }
 
-static GList *zephyr_away_states()
+static GList *zephyr_away_states(struct gaim_connection *gc)
 {
 	GList *m = NULL;
 
@@ -813,15 +837,15 @@ static GList *zephyr_chat_info(struct gaim_connection *gc) {
 
 	pce = g_new0(struct proto_chat_entry, 1);
 	pce->label = _("Class:");
-	m = g_list_append(m, NULL);
+	m = g_list_append(m, pce);
 
 	pce = g_new0(struct proto_chat_entry, 1);
 	pce->label = _("Instance:");
-	m = g_list_append(m, NULL);
+	m = g_list_append(m, pce);
 
 	pce = g_new0(struct proto_chat_entry, 1);
 	pce->label = _("Recipient:");
-	m = g_list_append(m, NULL);
+	m = g_list_append(m, pce);
 
 	return m;
 }
