@@ -41,7 +41,7 @@
 #define PROTO_NAPSTER	9
 #define PROTO_ZEPHYR   10
 #define PROTO_GADUGADU 11
-/* DON'T TAKE AN UNASSIGNED NUMBER! Talk to Eric or Rob if you'd like
+/* DON'T TAKE AN UNASSIGNED NUMBER! Talk to Rob or Sean if you'd like
  * to create a new PRPL. */
 
 #define PRPL_DESC(x)	"Allows gaim to use the " x " protocol.\n\n" \
@@ -55,7 +55,7 @@
 
 /* These should all be stuff that some plugins can do and others can't */
 /* TOC/Oscar send HTML-encoded messages; most other protocols don't */
-#define OPT_PROTO_HTML            0x00000001
+/* #define OPT_PROTO_HTML            0x00000001 this should be per-connection */
 /* TOC/Oscar have signon time, and the server's time needs to be adjusted to match
  * your computer's time. We wouldn't need this if everyone used NTP. */
 #define OPT_PROTO_CORRECT_TIME    0x00000002
@@ -70,22 +70,30 @@
 #define OPT_PROTO_MAIL_CHECK      0x00000020
 /* Oscar and Jabber have buddy icons */
 #define OPT_PROTO_BUDDY_ICON      0x00000040
+/* Oscar lets you send images in direct IMs */
+#define OPT_PROTO_IM_IMAGE        0x00000080
 
 #define GAIM_AWAY_CUSTOM "Custom"
 
 typedef void (*proto_init)(struct prpl *);
 
+struct _prpl_smiley {
+	char *key;
+	char **xpm;
+	int show;
+};
+
 struct prpl {
 	int protocol;
 	int options;
 	char *(* name)();
 
 	/* for ICQ and Yahoo, who have off/on per-conversation options */
-	char *checkbox;
+	/* char *checkbox; this should be per-connection */
 
 	/* returns the XPM associated with the given user class */
 	char **(* list_icon)(int);
-	GList *(* away_states)();
+	GList *(* away_states)(struct gaim_connection *gc);
 	GList *(* actions)();
 	void   (* do_action)(struct gaim_connection *, char *);
 	/* user_opts returns a GList* of g_malloc'd struct proto_user_opts */
@@ -93,6 +101,8 @@ struct prpl {
 	GList *(* buddy_menu)(struct gaim_connection *, char *);
 	GList *(* chat_info)(struct gaim_connection *);
 
+	GSList *(* smiley_list)();
+
 	/* all the server-related functions */
 
 	/* a lot of these (like get_dir) are protocol-dependent and should be removed. ones like
@@ -102,10 +112,12 @@ struct prpl {
 
 	void (* login)		(struct aim_user *);
 	void (* close)		(struct gaim_connection *);
-	int  (* send_im)	(struct gaim_connection *, char *who, char *message, int away);
+	int  (* send_im)	(struct gaim_connection *, char *who, char *message, int len, int away);
 	void (* set_info)	(struct gaim_connection *, char *info);
+	int  (* send_typing)    (struct gaim_connection *, char *name, int typing);
 	void (* get_info)	(struct gaim_connection *, char *who);
 	void (* set_away)	(struct gaim_connection *, char *state, char *message);
+	void (* get_away)       (struct gaim_connection *, char *who);
 	void (* set_dir)	(struct gaim_connection *, char *first,
 							   char *middle,
 							   char *last,
@@ -127,8 +139,8 @@ struct prpl {
 	void (* change_passwd)	(struct gaim_connection *, char *old, char *new);
 	void (* add_buddy)	(struct gaim_connection *, char *name);
 	void (* add_buddies)	(struct gaim_connection *, GList *buddies);
-	void (* remove_buddy)	(struct gaim_connection *, char *name);
-	void (* remove_buddies)	(struct gaim_connection *, GList *buddies);
+	void (* remove_buddy)	(struct gaim_connection *, char *name, char *group);
+	void (* remove_buddies)	(struct gaim_connection *, GList *buddies, char *group);
 	void (* add_permit)	(struct gaim_connection *, char *name);
 	void (* add_deny)	(struct gaim_connection *, char *name);
 	void (* rem_permit)	(struct gaim_connection *, char *name);
@@ -142,6 +154,21 @@ struct prpl {
 	int  (* chat_send)	(struct gaim_connection *, int id, char *message);
 	void (* keepalive)	(struct gaim_connection *);
 
+	/* new user registration */
+	void (* register_user)	(struct aim_user *);
+
+	/* get "chat buddy" info and away message */
+	void (* get_cb_info)	(struct gaim_connection *, int, char *who);
+	void (* get_cb_away)	(struct gaim_connection *, int, char *who);
+
+	/* save/store buddy's alias on server list/roster */
+	void (* alias_buddy)	(struct gaim_connection *, char *who);
+
+	/* change a buddy's group on a server list/roster */
+	void (* group_buddy)	(struct gaim_connection *, char *who, char *old_group, char *new_group);
+
+	void (* buddy_free)	(struct buddy *);
+
 	/* this is really bad. */
 	void (* convo_closed)   (struct gaim_connection *, char *who);
 
@@ -160,12 +187,17 @@ extern void unload_protocol(struct prpl *);
 extern struct prpl *find_prpl(int);
 extern void do_proto_menu();
 
-extern void do_ask_dialog(const char *, void *, void *, void *);
-extern void do_prompt_dialog(const char *, void *, void *, void *);
+extern void show_got_added(struct gaim_connection *, const char *,
+			   const char *, const char *, const char *);
 
-extern void connection_has_mail(struct gaim_connection *, int, const char *, const char *);
+extern void do_ask_dialog(const char *, void *, void *, void *);
+extern void do_prompt_dialog(const char *, const char *, void *, void *, void *);
+
+extern void connection_has_mail(struct gaim_connection *, int, const char *, const char *, const char *);
 
 extern void set_icon_data(struct gaim_connection *, char *, void *, int);
 extern void *get_icon_data(struct gaim_connection *, char *, int *);
 
+extern GSList *add_smiley(GSList *, char *, char **, int) ;
+
 #endif /* _PRPL_H_ */
