@@ -31,25 +31,33 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <errno.h>
+#include "gtkimhtml.h"
 #include "prpl.h"
 #include "multi.h"
 #include "gaim.h"
+#ifdef USE_APPLET
+#include "applet.h"
+#endif
 
 #include "pixmaps/ok.xpm"
 #include "pixmaps/cancel.xpm"
+#include "pixmaps/tb_search.xpm"
 
 void serv_login(struct aim_user *user)
 {
 	struct prpl *p = find_prpl(user->protocol);
+	
 	if (user->gc != NULL)
 		return;
+
 	if (p && p->login) {
-		debug_printf("Logging in using %s\n", (*p->name)());
-		(*p->login)(user);
-	} else {
-		do_error_dialog(_("You cannot log this account in; you do not have "
-				  "the protocol it uses loaded, or the protocol does "
-				  "not have a login function."), _("Login Error"));
+		if (!strlen(user->password) && !(p->options & OPT_PROTO_NO_PASSWORD)) {
+			do_error_dialog(_("Please enter your password"), _("Signon Error"));
+			return;
+		}
+
+		debug_printf(PACKAGE " " VERSION " logging in %s using %s\n", user->username, p->name());
+		p->login(user);
 	}
 }
 
@@ -57,7 +65,7 @@ static gboolean send_keepalive(gpointer d)
 {
 	struct gaim_connection *gc = d;
 	if (gc->prpl && gc->prpl->keepalive)
-		(*gc->prpl->keepalive)(gc);
+		gc->prpl->keepalive(gc);
 	return TRUE;
 }
 
@@ -79,6 +87,7 @@ void serv_close(struct gaim_connection *gc)
 		struct conversation *b = gc->buddy_chats->data;
 		gc->buddy_chats = g_slist_remove(gc->buddy_chats, b);
 		b->gc = NULL;
+		update_buttons_by_protocol(b);
 	}
 
 	if (gc->idle_timer > 0)
@@ -88,7 +97,7 @@ void serv_close(struct gaim_connection *gc)
 	update_keepalive(gc, FALSE);
 
 	if (gc->prpl && gc->prpl->close)
-		(*gc->prpl->close)(gc);
+		gc->prpl->close(gc);
 
 	account_offline(gc);
 	destroy_gaim_conn(gc);
@@ -129,44 +138,59 @@ void serv_finish_login(struct gaim_connection *gc)
 	update_keepalive(gc, TRUE);
 }
 
+/* This should return the elapsed time in seconds in which Gaim will not send
+ * typing notifications.
+ * if it returns zero, it will not send any more typing notifications 
+ * typing is a flag - TRUE for typing, FALSE for stopped typing */
+int serv_send_typing(struct gaim_connection *g, char *name, int typing) {
+	if (g && g->prpl && g->prpl->send_typing)
+		return g->prpl->send_typing(g, name, typing);
+	else return 0;
+}
 
-
-int serv_send_im(struct gaim_connection *gc, char *name, char *message, int flags)
+int serv_send_im(struct gaim_connection *gc, char *name, char *message, int len, int flags)
 {
 	int val = -EINVAL;
 	if (gc->prpl && gc->prpl->send_im)
-		val = (*gc->prpl->send_im)(gc, name, message, flags);
+		val = gc->prpl->send_im(gc, name, message, len, flags);
 
 	if (!(flags & IM_FLAG_AWAY))
 		serv_touch_idle(gc);
 
+	serv_send_typing(gc, name, FALSE);
 	return val;
 }
 
 void serv_get_info(struct gaim_connection *g, char *name)
 {
 	if (g && g->prpl && g->prpl->get_info)
-		(*g->prpl->get_info)(g, name);
+		g->prpl->get_info(g, name);
+}
+
+void serv_get_away(struct gaim_connection *g, char *name)
+{
+	if (g && g->prpl && g->prpl->get_away)
+		g->prpl->get_away(g, name);
 }
 
 void serv_get_dir(struct gaim_connection *g, char *name)
 {
-	if (g && g->prpl && g->prpl->get_dir)
-		(*g->prpl->get_dir)(g, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->get_dir)
+		g->prpl->get_dir(g, name);
 }
 
 void serv_set_dir(struct gaim_connection *g, char *first, char *middle, char *last, char *maiden,
 		  char *city, char *state, char *country, int web)
 {
-	if (g && g->prpl && g->prpl->set_dir)
-		(*g->prpl->set_dir)(g, first, middle, last, maiden, city, state, country, web);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->set_dir)
+		g->prpl->set_dir(g, first, middle, last, maiden, city, state, country, web);
 }
 
 void serv_dir_search(struct gaim_connection *g, char *first, char *middle, char *last, char *maiden,
 		     char *city, char *state, char *country, char *email)
 {
-	if (g && g->prpl && g->prpl->dir_search)
-		(*g->prpl->dir_search)(g, first, middle, last, maiden, city, state, country, email);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->dir_search)
+		g->prpl->dir_search(g, first, middle, last, maiden, city, state, country, email);
 }
 
 
@@ -176,13 +200,13 @@ void serv_set_away(struct gaim_connection *gc, char *state, char *message)
 		char *buf = NULL;
 		if (message) {
 			buf = g_malloc(strlen(message) + 1);
-			if (gc->prpl->options & OPT_PROTO_HTML)
+			if (gc->flags & OPT_CONN_HTML)
 				strncpy(buf, message, strlen(message) + 1);
 			else
 				strncpy_nohtml(buf, message, strlen(message) + 1);
 		}
 
-		(*gc->prpl->set_away)(gc, state, buf);
+		gc->prpl->set_away(gc, state, buf);
 
 		plugin_event(event_away, gc, state, buf, 0);
 
@@ -207,80 +231,112 @@ void serv_set_away_all(char *message)
 
 void serv_set_info(struct gaim_connection *g, char *info)
 {
-	if (g->prpl && g->prpl->set_info) {
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->set_info) {
 		plugin_event(event_set_info, g, info, 0, 0);
-		(*g->prpl->set_info)(g, info);
+		g->prpl->set_info(g, info);
 	}
 }
 
 void serv_change_passwd(struct gaim_connection *g, char *orig, char *new)
 {
-	if (g->prpl && g->prpl->change_passwd)
-		(*g->prpl->change_passwd)(g, orig, new);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->change_passwd)
+		g->prpl->change_passwd(g, orig, new);
 }
 
 void serv_add_buddy(struct gaim_connection *g, char *name)
 {
-	if (g->prpl && g->prpl->add_buddy)
-		(*g->prpl->add_buddy)(g, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->add_buddy)
+		g->prpl->add_buddy(g, name);
 }
 
 void serv_add_buddies(struct gaim_connection *g, GList *buddies)
 {
-	if (g->prpl) {
+	if (g && g_slist_find(connections, g) && g->prpl) {
 		if (g->prpl->add_buddies)
-			(*g->prpl->add_buddies)(g, buddies);
+			g->prpl->add_buddies(g, buddies);
 		else if (g->prpl->add_buddy)
 			while (buddies) {
-				(*g->prpl->add_buddy)(g, buddies->data);
+				g->prpl->add_buddy(g, buddies->data);
 				buddies = buddies->next;
 			}
 	}
 }
 
 
-void serv_remove_buddy(struct gaim_connection *g, char *name)
+void serv_remove_buddy(struct gaim_connection *g, char *name, char *group)
 {
-	if (g->prpl && g->prpl->remove_buddy)
-		(*g->prpl->remove_buddy)(g, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->remove_buddy)
+		g->prpl->remove_buddy(g, name, group);
 }
 
-void serv_remove_buddies(struct gaim_connection *gc, GList *g)
+void serv_remove_buddies(struct gaim_connection *gc, GList *g, char *group)
 {
+	if (!g_slist_find(connections, gc))
+		return;
 	if (!gc->prpl)
 		return;		/* how the hell did that happen? */
 	if (gc->prpl->remove_buddies)
-		(*gc->prpl->remove_buddies)(gc, g);
+		gc->prpl->remove_buddies(gc, g, group);
 	else {
 		while (g) {
-			serv_remove_buddy(gc, g->data);
+			serv_remove_buddy(gc, g->data, group);
 			g = g->next;
 		}
 	}
 }
 
+/*
+ * Set buddy's alias on server roster/list
+ */
+void serv_alias_buddy(struct buddy *b)
+{
+	if(b && b->gc && b->gc->prpl && b->gc->prpl->alias_buddy) {
+		b->gc->prpl->alias_buddy(b->gc, b->name);
+	}
+}
+
+/*
+ * Move a buddy from one group to another on server.
+ *
+ * Note: For now we'll not deal with changing gc's at the same time, but
+ * it should be possible.  Probably needs to be done, someday.
+ */
+void serv_move_buddy(struct buddy *b, struct group *og, struct group *ng)
+{
+	if(b && b->gc && og && og->gc && ng && ng->gc) {
+		/*
+		 * If there are no connection changes...
+		 */
+		if(b->gc == og->gc && b->gc == ng->gc && ng->gc == og->gc) {
+			if(b->gc->prpl && b->gc->prpl->group_buddy) {
+				b->gc->prpl->group_buddy(b->gc, b->name, og->name, ng->name);
+			}
+		}
+	}
+}
+
 void serv_add_permit(struct gaim_connection *g, char *name)
 {
-	if (g->prpl && g->prpl->add_permit)
-		(*g->prpl->add_permit)(g, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->add_permit)
+		g->prpl->add_permit(g, name);
 }
 
 void serv_add_deny(struct gaim_connection *g, char *name)
 {
-	if (g->prpl && g->prpl->add_deny)
-		(*g->prpl->add_deny)(g, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->add_deny)
+		g->prpl->add_deny(g, name);
 }
 
 void serv_rem_permit(struct gaim_connection *g, char *name)
 {
-	if (g->prpl && g->prpl->rem_permit)
-		(*g->prpl->rem_permit)(g, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->rem_permit)
+		g->prpl->rem_permit(g, name);
 }
 
 void serv_rem_deny(struct gaim_connection *g, char *name)
 {
-	if (g->prpl && g->prpl->rem_deny)
-		(*g->prpl->rem_deny)(g, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->rem_deny)
+		g->prpl->rem_deny(g, name);
 }
 
 void serv_set_permit_deny(struct gaim_connection *g)
@@ -288,61 +344,59 @@ void serv_set_permit_deny(struct gaim_connection *g)
 	/* this is called when either you import a buddy list, and make lots of changes that way,
 	 * or when the user toggles the permit/deny mode in the prefs. In either case you should
 	 * probably be resetting and resending the permit/deny info when you get this. */
-	if (g->prpl && g->prpl->set_permit_deny)
-		(*g->prpl->set_permit_deny)(g);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->set_permit_deny)
+		g->prpl->set_permit_deny(g);
 }
 
 
 void serv_set_idle(struct gaim_connection *g, int time)
 {
-	if (g->prpl && g->prpl->set_idle)
-		(*g->prpl->set_idle)(g, time);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->set_idle)
+		g->prpl->set_idle(g, time);
 }
 
 void serv_warn(struct gaim_connection *g, char *name, int anon)
 {
-	if (g->prpl && g->prpl->warn)
-		(*g->prpl->warn)(g, name, anon);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->warn)
+		g->prpl->warn(g, name, anon);
 }
 
 void serv_join_chat(struct gaim_connection *g, GList *data)
 {
-	if (g->prpl && g->prpl->join_chat)
-		(*g->prpl->join_chat)(g, data);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->join_chat)
+		g->prpl->join_chat(g, data);
 }
 
 void serv_chat_invite(struct gaim_connection *g, int id, char *message, char *name)
 {
 	char *buffy = message && *message ? g_strdup(message) : NULL;
 	plugin_event(event_chat_send_invite, g, (void *)id, name, &buffy);
-	if (g->prpl && g->prpl->chat_invite)
-		(*g->prpl->chat_invite)(g, id, buffy, name);
+	if (g && g_slist_find(connections, g) && g->prpl && g->prpl->chat_invite)
+		g->prpl->chat_invite(g, id, buffy, name);
 	if (buffy)
 		g_free(buffy);
 }
 
 void serv_chat_leave(struct gaim_connection *g, int id)
 {
-	/* i think this is the only one this should be necessary for since this is the
-	 * only thing that could possibly get called after the connection is closed */
 	if (!g_slist_find(connections, g))
 		return;
 
 	if (g->prpl && g->prpl->chat_leave)
-		(*g->prpl->chat_leave)(g, id);
+		g->prpl->chat_leave(g, id);
 }
 
 void serv_chat_whisper(struct gaim_connection *g, int id, char *who, char *message)
 {
 	if (g->prpl && g->prpl->chat_whisper)
-		(*g->prpl->chat_whisper)(g, id, who, message);
+		g->prpl->chat_whisper(g, id, who, message);
 }
 
 int serv_chat_send(struct gaim_connection *g, int id, char *message)
 {
 	int val = -EINVAL;
 	if (g->prpl && g->prpl->chat_send)
-		val = (*g->prpl->chat_send)(g, id, message);
+		val = g->prpl->chat_send(g, id, message);
 	serv_touch_idle(g);
 	return val;
 }
@@ -383,6 +437,11 @@ int find_queue_total_by_name(char *name)
 	return i;
 }
 
+struct queued_away_response {
+	char name[80];
+	time_t sent_away;
+};
+
 struct queued_away_response *find_queued_away_response_by_name(char *name)
 {
 	GSList *templist;
@@ -403,7 +462,7 @@ struct queued_away_response *find_queued_away_response_by_name(char *name)
 }
 
 /* woo. i'm actually going to comment this function. isn't that fun. make sure to follow along, kids */
-void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32 flags, time_t mtime)
+void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32 flags, time_t mtime, gint len)
 {
 	char *buffy;
 	char *angel;
@@ -433,29 +492,35 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 
 	/* plugin stuff. we pass a char ** but we don't want to pass what's been given us
 	 * by the prpls. so we create temp holders and pass those instead. it's basically
-	 * just to avoid segfaults. */
-	buffy = g_malloc(MAX(strlen(message) + 1, BUF_LONG));
-	strcpy(buffy, message);
-	angel = g_strdup(name);
-	plugin_return = plugin_event(event_im_recv, gc, &angel, &buffy, (void *)flags);
+	 * just to avoid segfaults. of course, if the data is binary, plugins don't see it.
+	 * bitch all you want; i really don't want you to be dealing with it. */
+	if (len < 0) {
+		buffy = g_malloc(MAX(strlen(message) + 1, BUF_LONG));
+		strcpy(buffy, message);
+		angel = g_strdup(name);
+		plugin_return = plugin_event(event_im_recv, gc, &angel, &buffy, (void *)&flags);
 
-	if (!buffy || !angel || plugin_return) {
-		if (buffy)
-			g_free(buffy);
-		if (angel)
-			g_free(angel);
-		return;
+		if (!buffy || !angel || plugin_return) {
+			if (buffy)
+				g_free(buffy);
+			if (angel)
+				g_free(angel);
+			return;
+		}
+		name = angel;
+		message = buffy;
+	} else {
+		name = g_strdup(name);
+		message = g_memdup(message, len);
 	}
-	name = angel;
-	message = buffy;
 
 	/* TiK, using TOC, sends an automated message in order to get your away message. Now,
 	 * this is one of the biggest hacks I think I've seen. But, in order to be nice to
 	 * TiK, we're going to give users the option to ignore it. */
-	if ((away_options & OPT_AWAY_TIK_HACK) && gc->away && strlen(gc->away) &&
+	if ((away_options & OPT_AWAY_TIK_HACK) && gc->away && strlen(gc->away) && (len < 0) &&
 	    !strcmp(message, ">>>Automated Message: Getting Away Message<<<")) {
 		char *tmpmsg = stylize(awaymessage->message, MSG_LEN);
-		serv_send_im(gc, name, tmpmsg, IM_FLAG_AWAY);
+		serv_send_im(gc, name, tmpmsg, -1, IM_FLAG_AWAY);
 		g_free(tmpmsg);
 		g_free(name);
 		g_free(message);
@@ -464,7 +529,7 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 
 	/* if you can't figure this out, stop reading right now.
 	 * "we're not worthy! we're not worthy!" */
-	if (convo_options & OPT_CONVO_SEND_LINKS)
+	if ((len < 0) && (convo_options & OPT_CONVO_SEND_LINKS))
 		linkify_text(message);
 
 	/* um. when we call write_to_conv with the message we received, it's nice to pass whether
@@ -499,12 +564,17 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 
 			qm = g_new0(struct queued_message, 1);
 			g_snprintf(qm->name, sizeof(qm->name), "%s", name);
-			qm->message = g_strdup(message);
+			qm->message = g_memdup(message, len == -1 ? strlen(message) + 1 : len);
 			qm->gc = gc;
 			qm->tm = mtime;
 			qm->flags = WFLAG_RECV | away;
+			qm->len = len;
 			message_queue = g_slist_append(message_queue, qm);
 
+#ifdef USE_APPLET
+			set_user_state(away);
+#endif
+
 			row = find_queue_row_by_name(qm->name);
 
 			if (row >= 0) {
@@ -540,16 +610,17 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 				set_convo_gc(cnv, gc);
 			}
 			if (new_conv && (sound_options & OPT_SOUND_FIRST_RCV))
-				play_sound(FIRST_RECEIVE);
-			else if (cnv->makesound && (sound_options & OPT_SOUND_RECV))
-				play_sound(RECEIVE);
+				play_sound(SND_FIRST_RECEIVE);
+			else if (cnv->makesound)
+				play_sound(SND_RECEIVE);
 
-			write_to_conv(cnv, message, away | WFLAG_RECV, NULL, mtime);
+			write_to_conv(cnv, message, away | WFLAG_RECV, NULL, mtime, len);
 		}
 
 		/* regardless of whether we queue it or not, we should send an auto-response. That is,
 		 * of course, unless the horse.... no wait. */
-		if ((away_options & OPT_AWAY_NO_AUTO_RESP) || !strlen(gc->away)) {
+		if ((away_options & OPT_AWAY_NO_AUTO_RESP) || !strlen(gc->away) ||
+				((away_options & OPT_AWAY_IDLE_RESP) && !gc->is_idle)) {
 			g_free(name);
 			g_free(message);
 			return;
@@ -567,7 +638,7 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 			qar->sent_away = 0;
 			away_time_queue = g_slist_append(away_time_queue, qar);
 		}
-		if ((t - qar->sent_away) < 120) {
+		if ((t - qar->sent_away) < away_resend) {
 			g_free(name);
 			g_free(message);
 			return;
@@ -576,7 +647,7 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 
 		/* apply default fonts and colors */
 		tmpmsg = stylize(gc->away, MSG_LEN);
-		serv_send_im(gc, name, away_subs(tmpmsg, alias), IM_FLAG_AWAY);
+		serv_send_im(gc, name, away_subs(tmpmsg, alias), -1, IM_FLAG_AWAY);
 		if (!cnv && clistqueue && (away_options & OPT_AWAY_QUEUE)) {
 			struct queued_message *qm;
 			qm = g_new0(struct queued_message, 1);
@@ -585,10 +656,11 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 			qm->gc = gc;
 			qm->tm = mtime;
 			qm->flags = WFLAG_SEND | WFLAG_AUTO;
+			qm->len = -1;
 			message_queue = g_slist_append(message_queue, qm);
 		} else if (cnv != NULL)
 			write_to_conv(cnv, away_subs(tmpmsg, alias), WFLAG_SEND | WFLAG_AUTO, NULL,
-				      mtime);
+				      mtime, len);
 		g_free(tmpmsg);
 	} else {
 		/* we're not away. this is easy. if the convo window doesn't exist, create and update
@@ -599,11 +671,13 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 			set_convo_gc(cnv, gc);
 		}
 		if (new_conv && (sound_options & OPT_SOUND_FIRST_RCV))
-			play_sound(FIRST_RECEIVE);
-		else if (cnv->makesound && (sound_options & OPT_SOUND_RECV))
-			play_sound(RECEIVE);
+			play_sound(SND_FIRST_RECEIVE);
+		else if (cnv->makesound)
+			play_sound(SND_RECEIVE);
 
-		write_to_conv(cnv, message, away | WFLAG_RECV, NULL, mtime);
+		set_convo_name(cnv, name);
+
+		write_to_conv(cnv, message, away | WFLAG_RECV, NULL, mtime, len);
 	}
 
 	plugin_event(event_im_displayed_rcvd, gc, name, message, (void *)flags);
@@ -614,13 +688,14 @@ void serv_got_im(struct gaim_connection *gc, char *name, char *message, guint32
 
 
 void serv_got_update(struct gaim_connection *gc, char *name, int loggedin, int evil, time_t signon,
-		     time_t idle, int type, gushort caps)
+		     time_t idle, int type, guint caps)
 {
 	struct buddy *b = find_buddy(gc, name);
 
-	if (gc->prpl->options & OPT_PROTO_CORRECT_TIME) {
+	if (signon && (gc->prpl->options & OPT_PROTO_CORRECT_TIME)) {
 		char *tmp = g_strdup(normalize(name));
 		if (!g_strcasecmp(tmp, normalize(gc->username))) {
+			gc->evil = evil;
 			gc->correction_time = (signon - gc->login_time);
 			update_idle_times();
 		}
@@ -636,40 +711,10 @@ void serv_got_update(struct gaim_connection *gc, char *name, int loggedin, int e
 	/* server with what's in our record.  We want to */
 	/* store things how THEY want it... */
 	if (strcmp(name, b->name)) {
-		GList *cnv = conversations;
-		struct conversation *cv;
-
-		char *who = g_malloc(80);
-
-		strcpy(who, normalize(name));
-
-		while (cnv) {
-			cv = (struct conversation *)cnv->data;
-			if (!g_strcasecmp(who, normalize(cv->name))) {
-				if (im_options & OPT_IM_ONE_WINDOW) {
-					set_convo_tab_label(cv, b->name);
-				} else {
-					g_snprintf(cv->name, sizeof(cv->name), "%s", name);
-					if (find_log_info(name) || (logging_options & OPT_LOG_ALL))
-						 g_snprintf(who, 63, LOG_CONVERSATION_TITLE, name);
-					else
-						g_snprintf(who, 63, CONVERSATION_TITLE, name);
-					gtk_window_set_title(GTK_WINDOW(cv->window), who);
-					/* was g_free(buf), but break gives us that
-					 * and freeing twice is not good --Sumner */
-					break;
-				}
-			}
-			cnv = cnv->next;
-		}
-		g_free(who);
-		who = g_strdup(b->name);
+		char *who = g_strdup(b->name);
 		g_snprintf(b->name, sizeof(b->name), "%s", name);
 		handle_buddy_rename(b, who);
 		g_free(who);
-
-		/* okay lets save the new config... */
-
 	}
 
 	if (!b->idle && idle) {
@@ -725,7 +770,7 @@ void serv_got_eviled(struct gaim_connection *gc, char *name, int lev)
 
 	plugin_event(event_warned, gc, name, (void *)lev, 0);
 
-	if (gc->evil > lev) {
+	if (gc->evil >= lev) {
 		gc->evil = lev;
 		return;
 	}
@@ -735,10 +780,32 @@ void serv_got_eviled(struct gaim_connection *gc, char *name, int lev)
 	g_snprintf(buf2, sizeof(buf2), "%s has just been warned by %s.\nYour new warning level is %d%%",
 		   gc->username, ((name == NULL)? "an anonymous person" : name), lev);
 
-	do_error_dialog(_("Warned"), buf2);
+	do_error_dialog(buf2, _("Warned"));
 }
 
+void serv_got_typing(struct gaim_connection *gc, char *name, int timeout) {
+	struct conversation *cnv = find_conversation(name);
+	 if (cnv) {
+		 set_convo_gc(cnv, gc);
+		 show_typing(cnv);
+	} else return;
+	 plugin_event(event_got_typing, gc, name, 0, 0);
+	 do_pounce(gc, name, OPT_POUNCE_TYPING);
+	 if (timeout > 0) {
+		 if (cnv->typing_timeout)
+			 gtk_timeout_remove (cnv->typing_timeout);
+		 cnv->typing_timeout = gtk_timeout_add(timeout * 1000,(GtkFunction)reset_typing,
+						       g_strdup(name));
+	 }
+}
 
+void serv_got_typing_stopped(struct gaim_connection *gc, char *name) {
+	struct conversation *c = find_conversation(name);
+	if (c && c->typing_timeout) {
+		gtk_timeout_remove (c->typing_timeout);
+	}
+	reset_typing(g_strdup(name));
+}
 
 static void close_invite(GtkWidget *w, GtkWidget *w2)
 {
@@ -840,9 +907,10 @@ struct conversation *serv_got_joined_chat(struct gaim_connection *gc, int id, ch
 	b->in_room = NULL;
 	b->id = id;
 	b->gc = gc;
+	b->send_history = g_list_append(NULL, NULL);
 	b->history = g_string_new("");
 	g_snprintf(b->name, 80, "%s", name);
-
+	
 	if ((logging_options & OPT_LOG_ALL) || find_log_info(b->name)) {
 		FILE *fd;
 		char *filename;
@@ -903,6 +971,8 @@ void serv_got_chat_in(struct gaim_connection *g, int id, char *who, int whisper,
 	GSList *bcs = g->buddy_chats;
 	struct conversation *b = NULL;
 	char *buf;
+	char *buffy, *angel;
+	int plugin_return;
 
 	while (bcs) {
 		b = (struct conversation *)bcs->data;
@@ -915,8 +985,26 @@ void serv_got_chat_in(struct gaim_connection *g, int id, char *who, int whisper,
 	if (!b)
 		return;
 
-	if (plugin_event(event_chat_recv, g, (void *)b->id, who, message))
+	
+	/* plugin stuff. we pass a char ** but we don't want to pass what's been given us
+	 * by the prpls. so we create temp holders and pass those instead. it's basically
+	 * just to avoid segfaults. of course, if the data is binary, plugins don't see it.
+	 * bitch all you want; i really don't want you to be dealing with it. */
+	
+	buffy = g_malloc(MAX(strlen(message) + 1, BUF_LONG));
+	strcpy(buffy, message);
+	angel = g_strdup(who);
+	plugin_return = plugin_event(event_chat_recv, g, (void *)b->id, &angel, &buffy);
+
+	if (!buffy || !angel || plugin_return) {
+		if (buffy)
+			g_free(buffy);
+		if (angel)
+			g_free(angel);
 		return;
+	}
+	who = angel;
+	message = buffy;
 
 	buf = g_malloc(MAX(strlen(message) * 2, 8192));
 	strcpy(buf, message);
@@ -930,5 +1018,66 @@ void serv_got_chat_in(struct gaim_connection *g, int id, char *who, int whisper,
 		w = 0;
 
 	chat_write(b, who, w, buf, mtime);
+	g_free(who);
+	g_free(message);
 	g_free(buf);
 }
+
+static void des_popup(GtkWidget *w, GtkWidget *window)
+{
+	if (w == window) {
+		char *u = gtk_object_get_user_data(GTK_OBJECT(window));
+		g_free(u);
+	}
+	gtk_widget_destroy(window);
+}
+
+void serv_got_popup(char *msg, char *u, int wid, int hei)
+{
+	GtkWidget *window;
+	GtkWidget *vbox;
+	GtkWidget *sw;
+	GtkWidget *text;
+	GtkWidget *hbox;
+	GtkWidget *button;
+	char *url = g_strdup(u);
+
+	GAIM_DIALOG(window);
+	gtk_window_set_wmclass(GTK_WINDOW(window), "popup", "Gaim");
+	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, TRUE);
+	gtk_window_set_title(GTK_WINDOW(window), "Gaim - Popup");
+	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
+	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(des_popup), window);
+	gtk_object_set_user_data(GTK_OBJECT(window), url);
+	gtk_widget_realize(window);
+	aol_icon(window->window);
+
+	vbox = gtk_vbox_new(FALSE, 5);
+	gtk_container_add(GTK_CONTAINER(window), vbox);
+
+	sw = gtk_scrolled_window_new(NULL, NULL);
+	gtk_widget_set_usize(sw, wid, hei);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+	gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 5);
+
+	text = gtk_imhtml_new(NULL, NULL);
+	gtk_container_add(GTK_CONTAINER(sw), text);
+	GTK_LAYOUT(text)->hadjustment->step_increment = 10.0;
+	GTK_LAYOUT(text)->vadjustment->step_increment = 10.0;
+	gaim_setup_imhtml(text);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+
+	button = picture_button(window, _("Close"), cancel_xpm);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(des_popup), window);
+
+	button = picture_button(window, _("More Info"), tb_search_xpm);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(open_url), url);
+
+	gtk_widget_show_all(window);
+
+	gtk_imhtml_append_text(GTK_IMHTML(text), msg, -1, GTK_IMHTML_NO_NEWLINE);
+}
