@@ -52,6 +52,23 @@
 #include "sounds/Receive.h"
 #include "sounds/RedAlert.h"
 
+/* label and opt are null for the buddy pounce because it's configured *
+ * per pounce. NULL option means it doesn't get displayed in the sound *
+ * preferences box */
+struct sound_struct sounds[NUM_SOUNDS] = {
+	N_("Buddy logs in"), OPT_SOUND_LOGIN, BuddyArrive, sizeof(BuddyArrive),
+	N_("Buddy logs out"), OPT_SOUND_LOGOUT, BuddyLeave, sizeof(BuddyLeave),
+	N_("Message received"), OPT_SOUND_RECV, Receive, sizeof(Receive),
+	N_("Message received begins conversation"), OPT_SOUND_FIRST_RCV, Receive, sizeof(Receive),
+	N_("Message sent"), OPT_SOUND_SEND, Send, sizeof(Send),
+	N_("Person enters chat"), OPT_SOUND_CHAT_JOIN, BuddyArrive, sizeof(BuddyArrive),
+	N_("Person leaves chat"), OPT_SOUND_CHAT_PART, BuddyLeave, sizeof(BuddyLeave),
+	N_("You talk in chat"), OPT_SOUND_CHAT_YOU_SAY, Send, sizeof(Send),
+	N_("Others talk in chat"), OPT_SOUND_CHAT_SAY, Receive, sizeof(Receive),
+	NULL, 0, RedAlert, sizeof(RedAlert),
+	N_("Someone says your name in chat"), OPT_SOUND_CHAT_NICK, Receive, sizeof(Receive)
+};
+
 static int check_dev(char *dev)
 {
 	struct stat stat_buf;
@@ -82,7 +99,7 @@ static void play_audio(unsigned char *data, int size)
 {
 	int fd;
 
-	fd = open("/dev/audio", O_WRONLY | O_EXCL);
+	fd = open("/dev/audio", O_WRONLY | O_EXCL | O_NDELAY);
 	if (fd < 0)
 		return;
 	write(fd, data, size);
@@ -106,7 +123,7 @@ static void play_audio_file(char *file)
 	read(fd, buf, info.st_size - 24);
 	close(fd);
 
-	fd = open("/dev/audio", O_WRONLY | O_EXCL);
+	fd = open("/dev/audio", O_WRONLY | O_EXCL | O_NDELAY);
 	if (fd < 0) {
 		free(buf);
 		return;
@@ -247,33 +264,39 @@ static int artsc_play_file(char *file)
 	int result = 0;
 	int fd = -1;
 
+	if (!can_play_artsc())
+		return 0;
+
 	fd = open(file, O_RDONLY);
 	if (fd < 0)
-		goto out;
+		return 0;
 
-	if (!can_play_artsc())
-		goto out;
+	if (fstat(fd, &stat_buf)) {
+		close(fd);
+		return 0;
+	}
 
-	if (fstat(fd, &stat_buf))
-		goto out;
-
-	if (!stat_buf.st_size)
-		goto out;
+	if (!stat_buf.st_size) {
+		close(fd);
+		return 0;
+	}
 
 	buf = g_malloc(stat_buf.st_size);
-	if (!buf)
-		goto out;
+	if (!buf) {
+		close(fd);
+		return 0;
+	}
 
-	if (read(fd, buf, stat_buf.st_size) < 0)
-		goto out;
+	if (read(fd, buf, stat_buf.st_size) < 0) {
+		g_free(buf);
+		close(fd);
+		return 0;
+	}
 
 	result = play_artsc(buf, stat_buf.st_size);
 
-      out:
-	if (buf)
-		g_free(buf);
-	if (fd != -1)
-		close(fd);
+	g_free(buf);
+	close(fd);
 	return result;
 }
 
@@ -385,20 +408,28 @@ void play_file(char *filename)
 	return;
 #endif
 
+	if (sound_options & OPT_SOUND_BEEP) {
+		gdk_beep();
+		return;
+	}
+
+	else if (sound_options & OPT_SOUND_NORMAL) {
+		debug_printf("attempting to play audio file with internal method -- this is unlikely to work\n");
+	}
+
 	pid = fork();
 
 	if (pid < 0)
 		return;
 	else if (pid == 0) {
-		if (sound_options & OPT_SOUND_BEEP) {
-			g_print("\a");
-			_exit(0);
-		}
-
-		if (sound_cmd[0]) {
+		alarm(30);
+		
+		if ((sound_options & OPT_SOUND_CMD) && sound_cmd[0]) {
 			char *args[4];
 			char command[4096];
-			g_snprintf(command, sizeof command, sound_cmd, filename);
+
+			g_snprintf(command, sizeof(command), sound_cmd, filename);
+
 			args[0] = "sh";
 			args[1] = "-c";
 			args[2] = command;
@@ -407,28 +438,32 @@ void play_file(char *filename)
 			_exit(0);
 		}
 #ifdef ESD_SOUND
-		if (esd_play_file(NULL, filename, 1))
-			_exit(0);
+		else if (sound_options & OPT_SOUND_ESD) {
+			if (esd_play_file(NULL, filename, 1))
+				_exit(0);
+		}
 #endif
 
 #ifdef ARTSC_SOUND
-		if (artsc_play_file(filename))
-			_exit(0);
+		else if (sound_options & OPT_SOUND_ARTSC) {
+			if (artsc_play_file(filename))
+				_exit(0);
+		}
 #endif
 
 #ifdef NAS_SOUND
-		if (play_nas_file(filename))
-			_exit(0);
+		else if (sound_options & OPT_SOUND_NAS) {
+			if (play_nas_file(filename))
+				_exit(0);
+		}
 #endif
-
-		if (can_play_audio()) {
+		else if ((sound_options & OPT_SOUND_NORMAL) && 
+			 can_play_audio()) {
 			play_audio_file(filename);
 			_exit(0);
 		}
 
 		_exit(0);
-	} else {
-		g_timeout_add(100, clean_pid, NULL);
 	}
 }
 
@@ -440,49 +475,63 @@ void play(unsigned char *data, int size)
 	return;
 #endif
 
-	pid = fork();
+	if (sound_options & OPT_SOUND_BEEP) {
+		gdk_beep();
+		return;
+	}
 
+	else if ((sound_options & OPT_SOUND_CMD) && sound_cmd[0]) {
+		debug_printf("can't play internal sound with external command -- skipping\n");
+		return;
+	}
+
+	pid = fork();
+	
 	if (pid < 0)
 		return;
 	else if (pid == 0) {
-		if (sound_options & OPT_SOUND_BEEP) {
-			g_print("\a");
-			_exit(0);
-		}
+		alarm(30);
+		
 #ifdef ESD_SOUND
 		/* ESD is our player of choice.  Are we OK to
 		 * go there? */
-		if (can_play_esd()) {
-			if (play_esd(data, size))
-				_exit(0);
+		if (sound_options & OPT_SOUND_ESD) {
+			if (can_play_esd()) {
+				if (play_esd(data, size))
+					_exit(0);
+			}
 		}
 #endif
 
 #ifdef ARTSC_SOUND
 		/* ArtsC is the new second choice. */
-		if (can_play_artsc()) {
-			if (play_artsc(data, size))
-				_exit(0);
+		if (sound_options & OPT_SOUND_ARTSC) {
+			if (can_play_artsc()) {
+				if (play_artsc(data, size))
+					_exit(0);
+			}
 		}
 #endif
 
 #ifdef NAS_SOUND
 		/* NAS is our second choice setup. */
-		if (can_play_nas()) {
-			if (play_nas(data, size))
-				_exit(0);
+		if (sound_options & OPT_SOUND_NAS) {
+			if (can_play_nas()) {
+				if (play_nas(data, size))
+					_exit(0);
+			}
 		}
 #endif
 
 		/* Lastly, we can try just plain old /dev/audio */
-		if (can_play_audio()) {
-			play_audio(data, size);
-			_exit(0);
+		if (sound_options & OPT_SOUND_NORMAL) {
+			if (can_play_audio()) {
+				play_audio(data, size);
+				_exit(0);
+			}
 		}
 
 		_exit(0);
-	} else {
-		g_timeout_add(100, clean_pid, NULL);
 	}
 }
 
@@ -490,98 +539,23 @@ extern int logins_not_muted;
 
 void play_sound(int sound)
 {
-
 	if (awaymessage && !(sound_options & OPT_SOUND_WHEN_AWAY))
 		return;
 
-	switch (sound) {
-	case BUDDY_ARRIVE:
-		if ((sound_options & OPT_SOUND_LOGIN) && logins_not_muted) {
-			if (sound_file[BUDDY_ARRIVE]) {
-				play_file(sound_file[BUDDY_ARRIVE]);
-			} else {
-				play(BuddyArrive, sizeof(BuddyArrive));
-			}
-		}
-		break;
-	case BUDDY_LEAVE:
-		if (sound_options & OPT_SOUND_LOGOUT) {
-			if (sound_file[BUDDY_LEAVE]) {
-				play_file(sound_file[BUDDY_LEAVE]);
-			} else {
-				play(BuddyLeave, sizeof(BuddyLeave));
-			}
-		}
-		break;
-	case FIRST_RECEIVE:
-		if (sound_options & OPT_SOUND_FIRST_RCV) {
-			if (sound_file[FIRST_RECEIVE]) {
-				play_file(sound_file[FIRST_RECEIVE]);
-			} else {
-				play(Receive, sizeof(Receive));
-			}
-		}
-		break;
-	case RECEIVE:
-		if (sound_options & OPT_SOUND_RECV) {
-			if (sound_file[RECEIVE]) {
-				play_file(sound_file[RECEIVE]);
-			} else {
-				play(Receive, sizeof(Receive));
-			}
-		}
-		break;
-	case SEND:
-		if (sound_options & OPT_SOUND_SEND) {
-			if (sound_file[SEND]) {
-				play_file(sound_file[SEND]);
-			} else {
-				play(Send, sizeof(Send));
-			}
-		}
-		break;
-	case CHAT_JOIN:
-		if (sound_options & OPT_SOUND_CHAT_JOIN) {
-			if (sound_file[CHAT_JOIN]) {
-				play_file(sound_file[CHAT_JOIN]);
-			} else {
-				play(BuddyArrive, sizeof(BuddyArrive));
-			}
-		}
-		break;
-	case CHAT_LEAVE:
-		if (sound_options & OPT_SOUND_CHAT_PART) {
-			if (sound_file[CHAT_LEAVE]) {
-				play_file(sound_file[CHAT_LEAVE]);
-			} else {
-				play(BuddyLeave, sizeof(BuddyLeave));
-			}
-		}
-		break;
-	case CHAT_YOU_SAY:
-		if (sound_options & OPT_SOUND_CHAT_YOU_SAY) {
-			if (sound_file[CHAT_YOU_SAY]) {
-				play_file(sound_file[CHAT_YOU_SAY]);
-			} else {
-				play(Send, sizeof(Send));
-			}
-		}
-		break;
-	case CHAT_SAY:
-		if (sound_options & OPT_SOUND_CHAT_SAY) {
-			if (sound_file[CHAT_SAY]) {
-				play_file(sound_file[CHAT_SAY]);
-			} else {
-				play(Receive, sizeof(Receive));
-			}
-		}
-		break;
-	case POUNCE_DEFAULT:
-		if (sound_file[POUNCE_DEFAULT]) {
-			play_file(sound_file[POUNCE_DEFAULT]);
+	if ((sound == SND_BUDDY_ARRIVE) && !logins_not_muted)
+		return;
+
+	if (sound >= NUM_SOUNDS) {
+		debug_printf("sorry old fruit... can't say I know that sound: ", sound, "\n");
+		return;
+	}
+
+	/* check NULL for sounds that don't have an option, ie buddy pounce */
+	if ((sound_options & sounds[sound].opt) || (sounds[sound].opt == 0)) {
+		if (sound_file[sound]) {
+			play_file(sound_file[sound]);
 		} else {
-			play(RedAlert, sizeof(RedAlert));
+			play(sounds[sound].snd, sounds[sound].snd_size);
 		}
-		break;
 	}
 }
