@@ -288,8 +288,10 @@ gint linkify_text(char *text)
 
 			while (flag) {
 				if (badchar(*t)) {
-					if (*(t - 1) == '.')
+					if (*(t - 1) == '.') {
+						url_buf[t-c] = '\0';
 						t--;
+					}
 					cnt += g_snprintf(&text[cnt], 1024,
 							  "<A HREF=\"mailto:%s\">%s</A>", url_buf,
 							  url_buf);
@@ -460,7 +462,7 @@ FILE *open_system_log_file(char *name)
 
 char alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz" "0123456789+/";
 
-
+/* XXX Find bug */
 char *tobase64(const char *text)
 {
 	char *out = NULL;
@@ -1264,3 +1266,43 @@ time_t get_time(int year, int month, int day, int hour, int min, int sec)
 	tm.tm_sec = sec >= 0 ? sec : time(NULL) % 60;
 	return mktime(&tm);
 }
+
+/*
+ * Like mkstemp() but returns a file pointer, uses a pre-set template,
+ * uses the semantics of tempnam() for the directory to use and allocates
+ * the space for the filepath.
+ *
+ * Caller is responsible for closing the file and removing it when done,
+ * as well as freeing the space pointed-to by "path" with g_free().
+ *
+ * Returns NULL on failure and cleans up after itself if so.
+ */
+static const char *gaim_mkstemp_templ = {"gaimXXXXXX"};
+
+FILE *gaim_mkstemp(gchar **fpath)
+{
+	gchar *tmpdir;
+	int fd;
+	FILE *fp = NULL;
+
+	if((tmpdir = g_get_tmp_dir()) != NULL) {
+		if((*fpath = g_strdup_printf("%s/%s", tmpdir, gaim_mkstemp_templ)) != NULL) {
+			if((fd = mkstemp(*fpath)) == -1) {
+				debug_printf("Error: Couldn't make \"%s\", error: %d\n", *fpath, errno);
+			} else {
+				if((fp = fdopen(fd, "r+")) == NULL) {
+					close(fd);
+					debug_printf("Error: Couldn't fdopen(), error: %d\n", errno);
+				}
+			}
+			if(!fp) {
+				g_free(*fpath);
+				*fpath = NULL;
+			}
+		}
+	} else {
+		debug_printf("Error: g_get_tmp_dir() failed in gaim_mkstemp()!\n");
+	}
+
+	return fp;
+}
